#include <easy.h>

char *e_tools_unix_match_set(char *pat,char c);

/*****************************************************/
/* Test if the unix pattern pat matches the string s */
/* Pattern : # match anything.                       */
/*           . match one char                        */
/*           [ab-x] : match group of character       */
/* Return : 1 : match                                */
/*          0 : no match                             */
/*****************************************************/
int e_tools_unix_match(char *pat,char *s)
{ char c,pc;

  while((c = *s++))
  { pc = *pat++;

    switch(pc)
    { case 0   : return(0); break;
      case '.' : break; /* match anything */
      case '*' : if(e_tools_unix_match(pat,s-1)) return(1);   /* pat+1 s   */
                 if(e_tools_unix_match(pat,s)) return(1);   /* pat+1 s+1 */
                 if(e_tools_unix_match(pat-1,s)) return(1); /* pat   s+1 */
                 return(0);
      case '[' :  /* match set of character */
                  if(!(pat = e_tools_unix_match_set(pat,c))) return(0);
                  break;
      case '\\' : pc = *pat++; /* escape char */                  
      default: if(c != pc) return(0);
               break;
    }
  }
  if(*pat == 0) return(1); /* the 2 eos match */ 
  if ((*pat == '*') && (pat[1] == 0)) return(1);
  return(0); /* no match */

}

/************************************/
/* Match a block of character       */
/* [abA-Z]: ab or A to Z            */
/* [-z...] : 0 to z ...             */
/* [a-] : a to 255                  */
/* return : 0 no match              */
/* pat pointer just after ] : match */
/************************************/        
char *e_tools_unix_match_set(char *pat,char c)
{ char pc,last,loop;

  last = 0;
  loop = 1;
  while(loop && (pc = *pat++))
  { switch(pc)
    { case ']' : return(0); /* no match */
      case '-' : if ((*pat==0)||(*pat == ']'))
                 { if(last <= c) loop = 0;
                 }
                 else if ((last <= c) && (c <= *pat)) 
                 { loop = 0; 
                 }
                 break;
      case '\\' : pc = *pat++;
      default : if (pc == c) 
                { loop = 0;
                }
    }
    last = pc;
  }
  if(loop) return(0); /* no match */

  /* search for then end of block! */
  while ((pc = *pat++) && (pc != ']'));
  return(pc ? pat : pat-1);
}
