#ifndef E_ASET_H
#define E_ASET_H
#include <stdio.h>

extern e_obj_descriptor e_obj_descriptor_aset;
#define E_IS_ASET (E_OBJ_DESCR(P) == e_obj_descriptor_aset)
#define E_TYPE_ASET (&e_obj_descriptor_aset)

extern e_obj_descriptor e_obj_descriptor_aset_item;
#define E_IS_ASET_ITEM (E_OBJ_DESCR(P) == e_obj_descriptor_aset_item)
#define E_TYPE_ASET_ITEM (&e_obj_descriptor_aset_item)


#define E_ASET_ADD_MODE_UNIQUE_VAL 1
#define E_ASET_ADD_MODE_UNIQUE_KEY (1<<1)
#define E_ASET_ADD_MODE_NO_REPLACE (1<<2)

                  /* Property modes */
#define E_ASET_PROPERTY_ADD_SET 1
#define E_ASET_PROPERTY_MULTI   2

typedef struct e_AsItem {
  e_obj h;           /* object header                   */
  struct e_AsItem *next; /* Next item                   */
  struct e_AsItem *prev; /* Previous item object        */
  e_name *name;          /* Item name                   */
  void *val;             /* item value                  */
} e_aset_item;


typedef struct {
  e_obj h;              /* object header                */
  e_aset_item *first;   /* First item                   */
  e_aset_item *last;    /* Last item                    */
  e_aset_item *cur;     /* Current item                 */
  int cardinal;         /* Number of item in the aset   */
} e_aset;

e_aset *e_aset_new(e_aset *asp);
e_aset_item *e_aset_item_new(e_aset_item *aip,e_name *np,void *val);
void e_aset_free(void *asp);
void e_aset_item_free(void *aip);
void *e_aset_to_string(void *asp,int tab,e_flag mode);
void *e_aset_item_to_string(void *aip,int tab,e_flag mode);
void *e_aset_clone(void *asp,e_flag mode);
void *e_aset_item_clone(void *aip,e_flag mode);
int e_aset_equals(void *p1,void *p2);
int e_aset_item_equals(void *p1,void *p2);
int e_aset_next(void *asp,int mode,void **obj);
e_aset *e_aset_append_item(e_aset *asp,e_name *np, void *val);
e_aset *e_aset_prepend_item(e_aset *asp,e_name *np, void *val);
e_aset *e_aset_add_item(e_aset *asp,e_name *np, void *val,int mode);
e_aset *e_aset_add_aset(e_aset *asp,e_aset *aspv,int mode);
e_aset *e_aset_append(e_aset *asp,e_aset *asp2);
int e_aset_get_key_index(e_aset *asp, e_name *np,int start);
int e_aset_get_element_index(e_aset *asp,void *obj,int start);
void *e_aset_get_value_by_key(e_aset *asp, e_name *np,int start);
void *e_aset_get_value_by_index(e_aset *asp,int index);
e_aset_item *e_aset_get_item_by_key(e_aset *asp, e_name *np);
e_aset_item *e_aset_get_item_by_value(e_aset *asp, void *val);
e_aset *e_aset_split(e_string *sp,char *sep, char *psep);
e_aset *e_aset_add_property(e_aset *asp, e_name *key, void *val,int mode);

#define E_ASET_FIRST(P) (((e_aset *)P)->first)
#define E_ASET_LAST(P) (((e_aset *)P)->last)
#define E_ASET_CUR(P) (((e_aset *)P)->cur)
#define E_ASET_CARDINAL(P) (((e_aset *)P)->cardinal)

#define E_ASET_ITEM_PREV(P) (((e_aset_item *)P)->prev)
#define E_ASET_ITEM_NEXT(P) (((e_aset_item *)P)->next)
#define E_ASET_ITEM_NAME(P) (((e_aset_item *)P)->name)
#define E_ASET_ITEM_VAL(P) (((e_aset_item *)P)->val)

#endif
