#ifndef E_CHARSTREAM_H
#define E_CHARSTREAM_H

extern e_obj_descriptor e_obj_descriptor_charstream;
#define E_IS_CHARSTREAM (E_OBJ_DESCR(P) == e_obj_descriptor_charstream)
#define E_TYPE_CHARSTREAM (&e_obj_descriptor_charstream)

                          /* Various type of recognised stream */
#define E_CHARSTREAM_TYPE_STRING 1
#define E_CHARSTREAM_TYPE_FILE   2
#define E_CHARSTREAM_TYPE_OTHER  3


typedef struct e_CStream {
  e_obj h;                        /* object header                           */
  void *obj;                      /* Pointer to object: e_file, e_string ... */
  void *stack;                    /* e_stack for unget                   ... */
  int type;                       /* E_CHARSTREAM_STREAM_STRING ...          */
  int mode;                       /* E_CHARSTREAM_MORE_READ,                 */
				  /* E_CHARSTREAM_MORE_WRITE                 */
  int index;                      /* Stream index position                   */
  int (* flush)(struct e_CStream *csp);
  int (* put)(struct e_CStream *csp,int c);
  int (* get)(struct e_CStream *csp);
  int (* unget)(struct e_CStream *csp,int c);
  int (* seek)(struct e_CStream *csp,long pos,int mode);
  long (* tell)(struct e_CStream *csp);
} e_charstream;


#define E_CHARSTREAM_OBJ(P) (((e_charstream *)P)->obj)
#define E_CHARSTREAM_STACK(P) (((e_charstream *)P)->stack)
#define E_CHARSTREAM_TYPE(P) (((e_charstream *)P)->type)
#define E_CHARSTREAM_MODE(P) (((e_charstream *)P)->mode)
#define E_CHARSTREAM_INDEX(P) (((e_charstream *)P)->index)

#define E_CHARSTREAM_FLUSH(P) (((e_charstream *)P)->flush(P))
#define E_CHARSTREAM_PUT(P,C) (((e_charstream *)P)->put(P,C))
#define E_CHARSTREAM_GET(P) (((e_charstream *)P)->get(P))
#define E_CHARSTREAM_UNGET(P,C) (((e_charstream *)P)->unget(P,C))
#define E_CHARSTREAM_SEEK(P,POS,M) (((e_charstream *)P)->seek(P,POS,M))
#define E_CHARSTREAM_TELL(P) (((e_charstream *)P)->tell(P))

e_charstream *e_charstream_new(e_charstream *ksp,int type,void *obj, int mode);
void e_charstream_free(void *ksp);
void *e_charstream_to_string(void *csp,int tab,e_flag mode);
void *e_charstream_clone(void *csp,e_flag mode);
int e_charstream_equals(void *p1,void *p2);
int e_charstream_unget(e_charstream *csp,int c);
long e_charstream_tell(e_charstream *csp);

int e_charstream_string_flush(e_charstream *csp);
int e_charstream_string_put(e_charstream *csp,int c);
int e_charstream_string_get(e_charstream *csp);
int e_charstream_string_seek(e_charstream *csp,long pos,int mode);
int e_charstream_file_flush(e_charstream *csp);
int e_charstream_file_put(e_charstream *csp,int c);
int e_charstream_file_get(e_charstream *csp);
int e_charstream_file_seek(e_charstream *csp,long pos,int mode);

#endif





