#ifndef E_ERROR_H
#define E_ERROR_H
#include <e_types.h>

#define E_LOG_SIZE 256

             /* Error Log Level*/
#define E_ERROR_LEVEL_PANIC     0
#define E_ERROR_LEVEL_USER      1
#define E_ERROR_LEVEL_VERBOSE   8
#define E_ERROR_LEVEL_ADMIN    12
#define E_ERROR_LEVEL_DEBUG    32

             /* Error Types */
#define E_ERROR_INCONSISTENCY   1
#define E_ERROR_MEMORY          (1<<1)
#define E_ERROR_NULL_POINTER    (1<<2) 
#define E_ERROR_PROTECTION      (1<<3) 
#define E_ERROR_INVALID_OBJECT  (1<<4)
#define E_ERROR_SYNTAX          (1<<5)
#define E_ERROR_FILE            (1<<6)
#define E_ERROR_ARITHMETIC      (1<<7) 
#define E_ERROR_OUT_OF_BOUND    (1<<8) 
#define E_ERROR_BAD_ARGUMENT    (1<<9) 

#define E_ERROR_ANY             (0x7fffffff) 


#define E_TEST_ERROR(ERR) (e_error_cur&ERR)

typedef struct {
  char text[E_LONG_LINE_SIZE];
  e_flag type;
  int log_level;
} e_error_log;

extern e_flag e_error_last;

int e_error_init(void *p);
void e_error(int type,int level,const char *fmt, ...);
void e_error_display(int level);
void *e_error_2_string(int level);
void e_reset();
void e_error_flush(int level);
void e_error_syntax(const char *fmt, ...);
int e_error_null_pointer(char *fct);
int e_error_no_memory(char *fct);
int e_error_is_writable(void *p,char *fct);
int e_error_is_readable(void *p,char *fct);
int e_check_type(void *p,e_obj_descriptor *type,char *fct);

#endif




