#ifndef E_TOKENISER_H
#define E_TOKENISER_H
#include <stdio.h>

extern e_obj_descriptor e_obj_descriptor_tokeniser;
#define E_IS_TOKENISER (E_OBJ_DESCR(P) == e_obj_descriptor_tokeniser)
#define E_TYPE_TOKENISER (&e_obj_descriptor_tokeniser)

#define E_TOKENISER_DEF_WORD   "_@a-zA-Z0-9"
#define E_TOKENISER_DEF_OP     "!#$%&*+--./<=>\\\\^?|~"
#define E_TOKENISER_DEF_PUNCT  "\"'(),:;`[]{}"
#define E_TOKENISER_DEF_SPACES " \a\b\t\v\f\r"

                         /* token types */
#define E_TOKENISER_WORD_TOKEN   1
#define E_TOKENISER_OP_TOKEN     2
#define E_TOKENISER_PUNCT_TOKEN  3
#define E_TOKENISER_STRING_TOKEN 4
#define E_TOKENISER_CHAR_TOKEN   5
#define E_TOKENISER_BIN_TOKEN    6
#define E_TOKENISER_EOL_TOKEN    7
#define E_TOKENISER_EOF_TOKEN    8

                         /* char types */
#define E_TOKENISER_WORD_CHAR   1
#define E_TOKENISER_OP_CHAR     2
#define E_TOKENISER_PUNCT_CHAR  3
#define E_TOKENISER_SPACE_CHAR  4
#define E_TOKENISER_BIN_CHAR    5

                         /* tokenising modes */
                         /* ignore lines starting with a # */
#define E_TOKENISER_MODE_HASH_COMMENT         1 
                         /* \ is the continuation character */
#define E_TOKENISER_MODE_SLASH_IS_CONT        (1<<1) 
                         /* \\ ignore the end of line */
#define E_TOKENISER_MODE_SLASH_SLASH_COMMENT  (1<<2) 
                         /* tokenise quoted strings */
#define E_TOKENISER_MODE_READ_STRING          (1<<3)  
                         /* tokenise quoted chars  */
#define E_TOKENISER_MODE_READ_CHAR            (1<<4)
                         /* tokenise binary character  */
#define E_TOKENISER_MODE_RETRUN_BIN           (1<<5)
                         /* tokenise end of line  */
#define E_TOKENISER_MODE_RETRUN_EOL           (1<<6) 

typedef struct {      
  e_obj h;             /* object header                   */
  e_charstream *csp;   /* charstream to read from         */
  char types[256];     /* type for each ASCII char        */
  char Buff[E_BUFF_SIZE]; /* Buffer for words    */
  e_string *sp;        /* quoted strings         */
  int mode;            /* parsing modes          */
  int type;            /* type of token read     */
  int newline;         /* 1 when new line starts */
  int line_no;         /* Line no in the stream  */
} e_tokeniser;

#define E_TOKENISER_CHARSTREAM(P) (((e_tokeniser *)P)->csp)
#define E_TOKENISER_TYPES(P)      (((e_tokeniser *)P)->types)
#define E_TOKENISER_BUFF(P)       (((e_tokeniser *)P)->Buff)
#define E_TOKENISER_STRING(P)     (((e_tokeniser *)P)->sp)
#define E_TOKENISER_LAST_CHAR(P)  (((e_tokeniser *)P)->last_char)
#define E_TOKENISER_MODE(P)       (((e_tokeniser *)P)->mode)
#define E_TOKENISER_TYPE(P)       (((e_tokeniser *)P)->type)
#define E_TOKENISER_NEWLINE(P)    (((e_tokeniser *)P)->newline)
#define E_TOKENISER_LINE_NO(P)    (((e_tokeniser *)P)->line_no)

#define E_TOKENISER_CHAR_IS_WORD(P,C) (E_TOKENISER_TYPES(P)[c] == E_TOKENISER_WORD_CHAR)
#define E_TOKENISER_CHAR_IS_OP(P,C) (E_TOKENISER_TYPES(P)[c] == E_TOKENISER_OP_CHAR)
#define E_TOKENISER_CHAR_IS_PUNCT(P,C) (E_TOKENISER_TYPES(P)[c] == E_TOKENISER_PUNCT_CHAR)
#define E_TOKENISER_CHAR_IS_SPACE(P,C) (E_TOKENISER_TYPES(P)[c] == E_TOKENISER_SPACE_CHAR)
#define E_TOKENISER_CHAR_IS_BIN(P,C) (E_TOKENISER_TYPES(P)[c] == E_TOKENISER_BIN_CHAR)

e_tokeniser *e_tokeniser_new(e_tokeniser *tp,e_charstream *csp,int mode,
                      char *word,char *op,char *punct, char *space);
void e_tokeniser_free(void *tp);
void *e_tokeniser_to_string(void *tp,int tab,e_flag mode);
void *e_tokeniser_clone(void *tp,e_flag mode);
int e_tokeniser_equals(void *p1,void *p2);
int e_tokeniser_next(e_tokeniser *tp);

int e_tokeniser_set_char_types(char *types,char *s,int t);
void e_tokeniser_char_types_to_buffer(char *types,char *buff,int t);
int e_tokeniser_escape_char(int c,char *s);


#endif

