#ifndef E_TYPES_H
#define E_TYPES_H
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <errno.h>

           /* Cloning  modes */
#define E_CLONE_RECURSIVE 1

#define E_TO_STR_COMPACT          1
#define E_TO_STR_QUOTE            2
#define E_TO_STR_EMBEDED          (E_TO_STR_COMPACT|E_TO_STR_QUOTE)
#define E_TO_STR_NOT_FIRST        4
#define E_TO_STR_NO_TAB_ON_FIRST  8
#define E_TO_STR_NO_TAB           (E_TO_STR_COMPACT|E_TO_STR_NO_TAB_ON_FIRST)

           /* Object protection modes */
#define E_OBJ_MODE_READ_ONLY  1
#define E_OBJ_MODE_NO_CLONE   2
#define E_OBJ_MODE_NO_READ    4

           /* Object scanning modes */
#define E_NEXT_GET_NEXT     0
#define E_NEXT_GET_CUR      1
#define E_NEXT_GET_NEXT_KEY 2
#define E_NEXT_GET_CUR_KEY  3
#define E_NEXT_RESET        4

           /* Various buffer sizes */
#define E_WORD_SIZE         20
#define E_LINE_SIZE         80
#define E_LONG_LINE_SIZE   256
#define E_PATH_SIZE        256
#define E_SMALL_BUFF_SIZE  256
#define E_BUFF_SIZE       1024
#define E_LONG_BUFF_SIZE  4096

           /* Various general modes */
#define E_MODE_MOVE_ABS          0
#define E_MODE_MOVE_REL          1
#define E_MODE_MOVE_END          2

#define E_MODE_READ          1
#define E_MODE_WRITE         2

           /* Atomic C Objects */
#define E_CTYPE_BOOLEAN   1
#define E_CTYPE_CHAR      2
#define E_CTYPE_INTEGER   3
#define E_CTYPE_REAL      4
#define E_CTYPE_COMP      5
#define E_CTYPE_STRING    6
#define E_CTYPE_FLAG      7


           /* Various integer types */
typedef long           e_int32;
typedef short          e_int16;
typedef char           e_int6;
typedef unsigned long  e_uint32;
typedef unsigned short e_uint16;
typedef unsigned char  e_uint8;
typedef double         e_real;
typedef long           e_size;
typedef unsigned long  e_flag;
typedef void *         e_pointer;
 

  /* This is the header of every object.                                */
  /*--------------------------------------------------------------------*/
typedef struct {

  void * obj_desc_p;       /* pointer to obj_descriptor       */

  e_int32 refs;            /* reference counter (-1 : static) */
                           /* Object mode :                   */ 
  e_flag  mode;            /*   read only, no cloning ...     */ 
} e_obj;


  /* The following structure has one static instance for each data type */
  /* The functions must be staticaly set to appropriate functions       */
  /*--------------------------------------------------------------------*/
typedef struct {
                                 /* Object type name               */
  char * Name;                   /* Constant string                */

  e_flag def_mode;               /* default mode for this object   */

                                 /* Raw Memory allocation function           */
  void *(* _alloc)(size_t size); /* maloc and free are examples              */
  void  (* _free)(void *p);      /* _free frees blocks allocated with _alloc */
  void *(* _realloc)(void *p,size_t size); /* increase the block size        */

                                /* Derefer object the object counter         */
  void (* free)(void * obj);    /* free (and close it) it if the counter is 0*/

                                /* Convert the object to a string.           */
                                /* Tab : identation, no of spaces to prepend */
                                /* mode: compact, ... */
  void *(* to_string)(void *obj,int tab,e_flag mode); 

                              /* Clone 1 object                 */
                              /* mode : E_CLONE_RECURSIVE recursively */
                              /* clone the object */
  void *(* clone)(void * obj,e_flag mode); 

                               /* Test if the 2 objects are identical */
                               /* return 1 if they are and 0 otherwise */
  int (* equals)(void *p1,void *p2); 

} e_obj_descriptor;


  /* Program descriptor:                                             */
  /* 4 fields to describe a program using constant strings           */
  /* The static variable e_Prog variable must be define as follow    */
  /* in the main program :                                           */ 
  /* static e_prog e_Prog = { "My Prog", "V1.0", "Me", "1/4/1999"};  */
  /*-----------------------------------------------------------------*/
typedef struct {
  char *name;
  char *version;
  char *author;
  char *date;
} e_prog;
 
/****************************** MACROS ************************/
#define E_OBJ(P) ((e_obj *) P)
#define E_OBJ_REF(P) (((e_obj *) P)->refs)
#define E_OBJ_MODE(P) (((e_obj *) P)->mode)
#define E_OBJ_DESCR(P) ((e_obj_descriptor *)((e_obj *) P)->obj_desc_p)
#define E_OBJ_NAME(P) (E_OBJ_DESCR(P)->Name)

#define E_USE(P) ((E_OBJ(P)->refs++),P)
#define E_REF(P) (E_OBJ(P)->refs++)
#define E_FREE(P) (E_OBJ_DESCR(P)->free(P))
#define E_CLONE(P,M) (E_OBJ_DESCR(P)->clone(P,M))
#define E_TO_STRING(P,T,M) (E_OBJ_DESCR(P)->to_string(P,T,M))
#define E_EQUALS(P1,P2) (E_OBJ_DESCR(P1)->equals(P1,P2))


#define E_OBJ_READ_ONLY(P) (E_OBJ_MODE(P) & E_OBJ_MODE_READ_ONLY)
#define E_OBJ_NO_CLONE(P) (E_OBJ_MODE(P) & E_OBJ_MODE_NO_CLONE)
#define E_OBJ_NO_READ(P) (E_OBJ_MODE(P) & E_OBJ_MODE_NO_READ)

#endif
