#include <sysf_init.h>
#include <sysf_com.h>
#include <ctype.h>
#include <unistd.h>
#include <sysf.h>


/**************************************************************/ 
/* sysf : Unix/Linux system file management system            */
/**************************************************************/ 

char VERSION[] = "sysf: v1.02; 6/6/2000;  Bernard Piette";

char HostName[E_LINE_SIZE];

/* Define some Text Constants */
char *Help[] = { 
"Syntax : sysf [PARAMETERS] [COMMAND] [HOSTS] [CONFIGS]",
" PARAMETERS:",
"  -echo=e    : echo host names and config files as they are being treated.",
"  -echo_host=eh  : echo host names as they are being treated.",
"  -mailto=m2 EMAIL_ADDRESS : Email the output to the specified address", 
"  -config_dir=cd CONFIG_DIR : config directory path.",
"  -config_file=cd CONFIG_FILE : config file name [sysf.conf].",
" COMMANDS:",
"  -install=I : install the config file (config.* -> sys)",
"  -first_install=FI : as install but make a .DIST copy of the system file.",
"  -OS_install=OSI : ignore SHARED files, first_install for all file but",
"             -new on files with h flag set when .HOST file does not exist."
"  -info=i    : display information about the config file",
"  -archive=A : archive the system file (sys -> config.*) if the system", 
"               and the config file differ. \".HOST\" config file only.", 
"  -new=N     : create a new config file (sys -> config.HOST)",
"               and create a sys.DIST file if it does not exist.",
"  -makedef=MD : create or update the DEFAULT configuration file.",
"  -makeshared=MS : create or update the SHARED configuration file.",
"  -makegroup=MG HOST_GROUP : create or update the HOST_GROUP configuration file.",
"  -uninstall=I : Execute the config command with the \"-Remove\" flag.",
"  -check=c   : check if the config file match the system file.",
"  -is_not_conf=nc : lists sys entries with no system or no config file",
"  -diff=d    : display the differences between the config and system files",
"  -diff_host=dh HOST : display the differences between the config files",
"               for this host and those listed in HOSTS.",
"  -check_host=ch HOST : check that the config files for this host and those",
"               listed in HOSTS are identical.",
"  -cat=C     : display the config file",
"  -catsys=Cs : display the system file",
"  -list_groups=lg : list the config groups",
"  -list_configs=lc : list all the config files by groups",
"  -list_1group=l1g GROUP: list the config files in that group",
"  -list_hosts=lh : list the host groups with their members",
"  -list_host_group=lhg : list the host members of the specified host group.",
" HOSTS:",
"  -hosts=h HOST1,HOST2,HOST_GROUP...",
" CONFIGS:",
"  CONFIG_NAMES : list of config names",
"  DIR_GROUPS : all config names in the specified group.",
"  ALL        : all the config names",

0};

struct {
  int install;              
  int first_install; 
  int OS_install;             
  int info;
  int archive;
  int new;
  int makedef;
  int makeshared;
  char makegroup[E_LINE_SIZE];
  int uninstall;              
  int check;
  int is_not_conf;
  int diff;
  char check_host[E_LINE_SIZE];
  char diff_host[E_LINE_SIZE];
  int cat;
  int catsys;
  int list_groups;
  int list_configs;
  char list_1group[E_LONG_LINE_SIZE];
  int list_hosts;
  char list_host_group[E_LONG_LINE_SIZE];
  char hosts[E_LONG_BUFF_SIZE];
  char mailto[E_LINE_SIZE];
  char config_dir[E_LONG_BUFF_SIZE];
  char config_file[E_LINE_SIZE];
  int echo;
  int echo_host;
} par;

struct sysf_par sysfile;

e_parameter Pars[] = {
{ "install",E_CTYPE_FLAG,&par.install,0,1,"I",0},   
{ "first_install",E_CTYPE_FLAG,&par.first_install,0,1,"FI",0},   
{ "OS_install",E_CTYPE_FLAG,&par.OS_install,0,1,"OSI",0},   
{ "info",E_CTYPE_FLAG,&par.info,0,1,"i",0},   
{ "archive",E_CTYPE_FLAG,&par.archive,0,1,"A",0},   
{ "new",E_CTYPE_FLAG,&par.new,0,1,"N",0},   
{ "makedef",E_CTYPE_FLAG,&par.makedef,0,1,"MD",0},   
{ "makeshared",E_CTYPE_FLAG,&par.makeshared,0,1,"MD",0},   
{ "makegroup",E_CTYPE_STRING,par.makegroup,"",0,"MG",0},   
{ "uninstall",E_CTYPE_FLAG,&par.uninstall,0,1,"U",0},   
{ "check",E_CTYPE_FLAG,&par.check,0,1,"c",0},   
{ "is_not_conf",E_CTYPE_FLAG,&par.is_not_conf,0,1,"nc",0},   
{ "diff",E_CTYPE_FLAG,&par.diff,0,1,"d",0},   
{ "check_host",E_CTYPE_STRING,par.check_host,"",0,"ch",0},   
{ "diff_host",E_CTYPE_STRING,par.diff_host,"",0,"dh",0},   
{ "cat",E_CTYPE_FLAG,&par.cat,0,1,"C",0},   
{ "catsys",E_CTYPE_FLAG,&par.catsys,0,1,"Cs",0},   
{ "list_groups",E_CTYPE_FLAG,&par.list_groups,0,1,"lg",0},   
{ "list_configs",E_CTYPE_FLAG,&par.list_configs,0,1,"lc",0},   
{ "list_1group",E_CTYPE_STRING,par.list_1group,"",0,"l1g",0},   
{ "list_hosts",E_CTYPE_FLAG,&par.list_hosts,0,1,"lh",0},   
{ "list_host_group",E_CTYPE_STRING,par.list_host_group,"",0,"lhg",0},   
{ "hosts",E_CTYPE_STRING,par.hosts,"",0,"h",0},   
{ "mailto",E_CTYPE_STRING,par.mailto,"",0,"m2",0},   
{ "config_dir",E_CTYPE_STRING,par.config_dir,SYSF_CONFIG_DIR,0,"cd",0},   
{ "config_file",E_CTYPE_STRING,par.config_file,SYSF_CONFIG_FILE,0,"cf",0},   
{ "echo",E_CTYPE_FLAG,&par.echo,0,1,"e",0},   
{ "echo_host",E_CTYPE_FLAG,&par.echo_host,0,1,"eh",0},   
{ 0,0,0,0,0,0},
};

int ExecSysf(int Argc, char **Argv,char *HostName);
void get_command_name(char *name);
int commands_no();

/**************************************************************************/
/*                        MAIN                                            */   
/**************************************************************************/
int main(argc, argv)
int argc;
char *argv[];
{ int i,n,Argc,err;
  char **Argv;
  e_string *sp,*arg_sp = 0,*spcom = 0;

  sysfile.messages = 0;
  sysfile.hosts_groups = 0;
  sysfile.CheckCounter = 0;
 
  E_NULL_ERROR(e_init());

           /* Reconstruct the argument list but without the hosts */
           /*******************************************************/
  arg_sp = e_string_new(0,0,E_LINE_SIZE);
  i = 1;
  while (i < argc)
  { if(!strcmp(argv[i],"-h") || !strcmp(argv[i],"-hosts"))
    { i++;   /* skip [-hosts=h] HOST_NAME */
    }
    else
    { arg_sp = e_string_append_buffer(arg_sp,argv[i]);
      arg_sp = e_string_append_buffer(arg_sp," ");
    }
    i++;
  }

           /* Read the argument list*/
           /*************************/
  if((Argc = e_parameter_read_arg_line(argc,argv,Pars,Help,&Argv,
                                       E_PARAMETER_V_IS_VERSION))<0)
  { goto error;
  }

           /* Default command : info */
           /**************************/
  n = commands_no();
  if(n > 1)
  { mprintf("Only one sysf command can be selected at a time.!\n");
    goto error;    
  }
  else if (n == 0)
  { par.info = 1; 
  }

  if(gethostname(HostName,E_LINE_SIZE))
  { mprintf("Unable to get host name!\n");
    if(sysfile.messages) mail(par.mailto,"sysf");
    goto error;
  }

  if(!par.hosts[0])       /* By default use the local host */
  { strncpy(par.hosts,HostName,E_LINE_SIZE-1);
    par.hosts[E_LINE_SIZE] = 0;
  } 


           /* Do we email the output?       */
           /********************************/
  if(par.mailto[0])
  { E_NULL_ERROR(sysfile.messages = e_string_new(0,"",E_LONG_BUFF_SIZE)); 
  }


           /* Read the configuration files */
           /********************************/
  E_NULL_ERROR(read_sysf_config(par.config_dir,par.config_file));

  E_NULL_ERROR(make_config_group_lists());

  E_NULL_ERROR(read_sysf_hosts(par.config_dir,"hosts.conf"));


            /* Execute the info commands, no HOSTS nor CONFIG required */ 
            /***********************************************************/
  if(par.list_groups)    { display_config_groups();            goto end; }
  if(par.list_configs)   { display_all_config_files();         goto end; }
  if(par.list_1group[0]) { display_one_group(par.list_1group); goto end; }   
  if(par.list_hosts)     { display_hosts_groups();             goto end; }
  if(par.list_host_group[0]) 
                         { display_host_group(par.list_host_group); 
                           goto end;
                         }

  if(Argc < 0)           { e_Syntax(Help);                     exit(0);  }

            /* Create the list of argument hosts, resolving hosts groups */ 
            /*************************************************************/
  E_NULL_ERROR(sysfile.hosts = read_args_hosts(par.hosts));

            /* Scan the list of hosts.                                 */ 
            /* For other hosts, execute sysf remotely.                 */ 
            /***********************************************************/
  e_aset_next(sysfile.hosts,E_NEXT_RESET,0);
  while(e_aset_next(sysfile.hosts,E_NEXT_GET_NEXT,(void **)&sp) > 0)
  { 
    if(!strcmp(E_STRING_S(sp),HostName))
    { E_NULL_ERROR(get_this_host_groups(sp));
      E_NULL_ERROR(ExecSysf(Argc,Argv,HostName));
    }
    else /* rsh HOST "sysf ARGLIST " */
    { E_NULL_ERROR(spcom = e_string_new(0,RSH_COMM" ",100));
      E_NULL_ERROR(spcom = e_string_append_buffer(spcom,E_STRING_S(sp)));
      E_NULL_ERROR(spcom = e_string_append_buffer(spcom," \"sysf "));
      E_NULL_ERROR(spcom = e_string_append_buffer(spcom,E_STRING_S(arg_sp)));
      E_NULL_ERROR(spcom = e_string_append_buffer(spcom," \""));

      if(!(spcom = e_exec_sh_command(E_STRING_S(spcom),&err))) goto error;
      mprintf("%s",E_STRING_S(spcom));
    }   
  }
end:
  if(spcom) E_FREE(spcom);
  if(arg_sp) E_FREE(arg_sp);

  if(par.mailto[0] && sysfile.messages) mail(par.mailto,"sysf");
 
  exit(0);

error:
  if(spcom) E_FREE(spcom);
  if(arg_sp) E_FREE(arg_sp);
  

  if(par.mailto[0] && sysfile.messages)
  { sysfile.messages = e_string_append_string(sysfile.messages,
                                            e_error_2_string(E_ERROR_ANY)); 
    mail(par.mailto,"sysf");
  }

                    /* Still echo error on stderr, even if mailed!  */
  e_error_flush(E_ERROR_ANY);

  exit(1);   
}


/*****************************/
/* Execute the sysf commands */
/* Return 1 : OK             */
/*        0 : error          */
/*****************************/
int ExecSysf(int Argc, char **Argv,char *HostName)
{ char *conf,ComName[E_LINE_SIZE];
  int i,NoCheck;
  e_aset *stp = 0,*grp,*conf_data,*asp = 0;
  e_name *np,*cnp;
  sysfile_info sysfip;
  file_info host_fi;

  NoCheck = par.new;               /* Don't check if config file exists  */
  get_command_name(ComName);

  if(par.echo || par.echo_host) 
  { mprintf("%s\n",HostName);
  }

     /* create an aset with all the argument config files                */
     /* sysfile.arg_configs : aset of e_name : one for each config file  */
     /* Config groups are translated into config file names              */
  E_NULL_ERROR(sysfile.arg_configs = e_aset_new(0));

  if(Argc == 0) /* No config name : use ALL */
  {             /* Add the set of names to arg_configs */
    e_aset_next(sysfile.configs,E_NEXT_RESET,0);
  
               /* Scan the list of all config files         */
    while(e_aset_next(sysfile.configs,E_NEXT_GET_CUR_KEY,(void **)&cnp) > 0)
    { e_aset_next(sysfile.configs,E_NEXT_GET_NEXT,(void **)&asp);
      if(asp) E_FREE(asp); asp = 0;

              /* Property values are the config names      */
      E_NULL_ERROR(sysfile.arg_configs = 
                         e_aset_add_item(sysfile.arg_configs,0,
                                         cnp,E_ASET_ADD_MODE_UNIQUE_VAL)); 
      cnp = 0; 
    }
  }
  else
  { for (i=0; i < Argc; i++)
    { E_NULL_ERROR(np = e_name_new(Argv[i]));

                      /* A config file */
      if((stp = e_aset_get_value_by_key(sysfile.configs,E_USE(np),0)))
      {                  /* Add the name to arg_configs */
        E_NULL_ERROR(sysfile.arg_configs = 
                e_aset_add_item(sysfile.arg_configs,0,np,
                                E_ASET_ADD_MODE_UNIQUE_VAL));
        np = 0;
        E_FREE(stp); stp = 0;
      }
                      /* A config group */
      else if((grp=e_aset_get_value_by_key(sysfile.config_groups,
                                           E_USE(np),0))) 
      {               /* Add the set of names to arg_configs */
        E_NULL_ERROR(sysfile.arg_configs = 
          e_aset_add_aset(sysfile.arg_configs,grp,E_ASET_ADD_MODE_UNIQUE_VAL));
        E_FREE(np); np = 0; grp = 0;
      }
                      /* ALL : All config files */
      else if(!strcmp("ALL",E_NAME_NAME(np))) 
      {               /* Add the set of names to arg_configs */
        e_aset_next(sysfile.configs,E_NEXT_RESET,0);
  
                      /* Scan the list of all config files         */
        while(e_aset_next(sysfile.configs,E_NEXT_GET_CUR_KEY,(void **)&cnp)>0)
        { e_aset_next(sysfile.configs,E_NEXT_GET_NEXT,(void **)&asp);
          if(asp) E_FREE(asp); asp = 0;

                      /* Property values are the config names      */
           E_NULL_ERROR(sysfile.arg_configs = 
                           e_aset_add_item(sysfile.arg_configs,0,
                                           cnp,E_ASET_ADD_MODE_UNIQUE_VAL));
           cnp = 0;
        }
      }
      else
      { 
        e_error(E_ERROR_BAD_ARGUMENT,E_ERROR_LEVEL_USER,
                "sysf error : invalid config file or group %s\n",Argv[i]);
        goto error;
      }
    }
  }

  e_aset_next(sysfile.arg_configs,E_NEXT_RESET,0);
  while(e_aset_next(sysfile.arg_configs,E_NEXT_GET_NEXT,(void **)&np) > 0)
  { conf = E_NAME_NAME(np);
    if(par.echo || par.info) mprintf("%s:\n",conf);

                      /* Get the data for the config file. */
    E_NULL_ERROR(conf_data = e_aset_get_value_by_key(sysfile.configs,
                                                     E_USE(np),0));

    E_NULL_ERROR(get_system_file(conf,conf_data,&sysfip.sys_file));

    if(par.echo) 
    { if(par.makegroup[0]) 
           mprintf("%s : %s %s : %s\n",HostName,ComName,par.makegroup,conf);
      else mprintf("%s : %s : %s\n",HostName,ComName,conf);
    };

                           /* get system flags  */
    E_NULL_ERROR(get_system_file_flags(conf,conf_data,&sysfip.flags));
    E_NULL_ERROR(get_system_command(conf,par.config_dir,conf_data,
                                    &sysfip.sys_com));    

    if(sysfip.flags & E_SYSF_FLAG_s)   
    {                      /* config file with GLOBAL suffix (no check) */
      E_NULL_ERROR(get_config_file(conf,par.config_dir,conf_data,
                                   &sysfip.config_file,"SHARED",1));
    }
    else if(par.makedef)   
    {                      /* config file with DEFAULT suffix (no check) */
      E_NULL_ERROR(get_config_file(conf,par.config_dir,conf_data,
                                   &sysfip.config_file,"DEFAULT",1));
    }
    else if(par.makeshared)   
    {                      /* config file with SHARED suffix (no check) */
      E_NULL_ERROR(get_config_file(conf,par.config_dir,conf_data,
                                   &sysfip.config_file,"SHARED",1));
    }
    else if(par.makegroup[0]) 
    {                      /* config file with HOST GROUP suffix (no check) */
      if(e_aset_get_key_index(sysfile.hosts_groups,
                              e_name_new(par.makegroup),0) < 0 )
      { e_error(E_ERROR_BAD_ARGUMENT,E_ERROR_LEVEL_USER,
              "Host Group %s does not exist",par.makegroup);
      }

      E_NULL_ERROR(get_config_file(conf,par.config_dir,conf_data,
                                   &sysfip.config_file,par.makegroup,1));
    }
    else
    {                      /* config file with HOST suffix (check) */
      get_config_file(conf,par.config_dir,conf_data,
		      &sysfip.config_file,HostName,NoCheck);
    }


    if ((sysfip.flags & E_SYSF_FLAG_C))
    {                      /* Create the system file */
      if((!execute_sys_com("-CreateSysFile",ComName,&sysfip,par.echo))) 
          goto error;
                           /* check that the system file has been created */
      E_NULL_ERROR(get_system_file(conf,conf_data,&sysfip.sys_file));
    }

    if(par.is_not_conf) 
    { E_NULL_ERROR(is_not_conf(conf,&sysfip,par.echo));      
    }; 
                           /* Check or compare two hosts */
    if(par.check_host[0]) 
    { if(!(sysfip.flags & E_SYSF_FLAG_h) && !(sysfip.flags & E_SYSF_FLAG_s))
      { get_config_file(conf,par.config_dir,conf_data,&host_fi,
                        par.check_host,0);

         E_NULL_ERROR(check_host(conf,&host_fi,par.check_host,&sysfip,HostName,
                                 par.echo));
      }
      continue;
    }; 

    if(par.diff_host[0]) 
    { if(!(sysfip.flags & E_SYSF_FLAG_s))
      { get_config_file(conf,par.config_dir,conf_data,&host_fi,
                        par.diff_host,0);
        E_NULL_ERROR(diff_host(conf,&host_fi,par.diff_host,&sysfip,HostName,
                               par.echo));
      }
      continue;
    }; 

                      /* test if the files are identical */
    test_sysfile(&sysfip);

    if(par.install || par.first_install || par.OS_install) 
    { if((sysfip.flags & E_SYSF_FLAG_I) &&
	(!execute_sys_com("-PreInstall","-install",&sysfip,par.echo))) 
          goto error;
        
      if(par.install)
      { E_NULL_ERROR(install(conf,&sysfip,par.echo)); 
      } 
      else if(par.first_install) 
      { E_NULL_ERROR(first_install(conf,&sysfip,par.echo));
      }
      else
      { /* OS_Install */
                /* Ignore shared files */
        if(!(sysfip.flags & E_SYSF_FLAG_s))
        {       /* by default do a first install if the config file exists */
          if(sysfip.config_file.exists)
          { first_install(conf,&sysfip,par.echo);
          }
          else
	  { if((sysfip.flags & E_SYSF_FLAG_h) && sysfip.sys_file.exists)
            {   /* no config file for h file: create it with new */
                /* Create the config file CONF.HOSTMAME          */
              get_config_file(conf,par.config_dir,conf_data,
		              &sysfip.config_file,HostName,1);
              new(conf,&sysfip,par.echo);
	    }
          }
        }
      }

      if((sysfip.flags & E_SYSF_FLAG_C) &&
        (!execute_sys_com("-UpdateSysFile","-install",&sysfip,par.echo))) 
           goto error;

      if((sysfip.flags & E_SYSF_FLAG_I) && 
        (!execute_sys_com("-PostInstall","-install",&sysfip,par.echo))) 
           goto error;
    }              

    if(par.uninstall) 
    { if((sysfip.flags & E_SYSF_FLAG_U) && 
         !execute_sys_com("-Remove","-uninstall",&sysfip,par.echo)) 
           goto error;
    }   

    if(par.info)  
    { E_NULL_ERROR(get_info(conf,&sysfip,par.echo));   
    }; 

    if(par.archive) 
    { E_NULL_ERROR(archive(conf,&sysfip,par.echo));  
    };  

    if(par.new)   
    { E_NULL_ERROR(new(conf,&sysfip,par.echo));        
    }; 

    if(par.makedef)   
    { E_NULL_ERROR(makedef(conf,&sysfip,par.echo));        
    }; 

    if(par.makeshared)   
    { E_NULL_ERROR(makedef(conf,&sysfip,par.echo));        
    }; 

    if(par.makegroup[0])   
    { E_NULL_ERROR(makedef(conf,&sysfip,par.echo));        
    }; 

    if(par.check) 
    { E_NULL_ERROR(check(conf,&sysfip,par.echo));      
    }; 

    if(par.diff)  
    { E_NULL_ERROR(diff(conf,&sysfip,par.echo));       
    }; 

    if(par.cat)   
    { E_NULL_ERROR(cat_config(conf,&sysfip,par.echo)); 
    }; 

    if(par.catsys)   
    { E_NULL_ERROR(cat_system(conf,&sysfip,par.echo)); 
    }; 
  }
  return(1);

error:
  if(stp) E_FREE(stp);
  return(0);
}

/****************************************/
/* Return the sysf command name in name */
/* name : buffer                        */
/****************************************/
void get_command_name(char *name)
{ 
  if(!name) return;
  name[0] = 0;
  if(par.makedef) strcpy(name,"makedef");
  if(par.makeshared) strcpy(name,"makeshared");
  if(par.makegroup[0]) strcpy(name,"makegroup");
  if(par.new) strcpy(name,"new");
  if(par.uninstall) strcpy(name,"uninstall");
  if(par.install) strcpy(name,"install");
  if(par.first_install) strcpy(name,"first_install");
  if(par.OS_install) strcpy(name,"OS_install");
  if(par.info) strcpy(name,"info");
  if(par.archive) strcpy(name,"archive");
  if(par.check) strcpy(name,"check");
  if(par.is_not_conf) strcpy(name,"is_not_conf");
  if(par.diff) strcpy(name,"diff");
  if(par.check_host[0]) strcpy(name,"check_host");
  if(par.diff_host[0])  strcpy(name,"diff_host");
  if(par.cat) strcpy(name,"cat");
  if(par.catsys) strcpy(name,"catsys");

}

/**********************************************/
/* Check the number of sysf command selected  */
/* There should be only one.                  */
/* If there is none, info will be the default */
/* return : the number of comand selected     */
/**********************************************/
int commands_no()
{ int no_com;

  no_com = 0;
  if(par.makedef)      no_com++; 
  if(par.makeshared)   no_com++; 
  if(par.makegroup[0]) no_com++; 
  if(par.new)          no_com++; 
  if(par.uninstall)    no_com++;
  if(par.install)      no_com++;
  if(par.first_install) no_com++; 
  if(par.OS_install)   no_com++; 
  if(par.info)         no_com++;
  if(par.archive)      no_com++; 
  if(par.check)        no_com++;
  if(par.is_not_conf)  no_com++;
  if(par.diff)         no_com++;
  if(par.check_host[0]) no_com++; 
  if(par.diff_host[0]) no_com++; 
  if(par.cat)          no_com++; 
  if(par.catsys)       no_com++;
  if(par.list_groups)     no_com++;
  if(par.list_configs)    no_com++;
  if(par.list_1group[0])  no_com++;
  if(par.list_hosts)      no_com++;
  if(par.list_host_group[0]) no_com++;

  return(no_com);
}
