#include <easy.h>
#include <stdio.h>
#include <sys/stat.h>
#include <unistd.h>
#include "sysf.h"
#include "sysf_com.h"
#include "sysf_init.h"

extern char HostName[E_LINE_SIZE];

/************************************************/
/* Display info about the selected config file  */
/* name : config name                           */
/* sip : pointer to sysfile_file info structure */
/************************************************/
int get_info(char *name,sysfile_info *sip,int echo)
{ char sys_time[E_LINE_SIZE],conf_time[E_LINE_SIZE];

  strftime(sys_time,E_LINE_SIZE,"%H:%M:%S %d/%m/%Y",
           localtime(&sip->sys_file.time));
  strftime(conf_time,E_LINE_SIZE,"%H:%M:%S %d/%m/%Y",
           localtime(&sip->config_file.time));

  if(!sip->sys_file.exists)
  { mprintf(" System file (%s) does not exist.\n",sip->sys_file.FileName);
  }
  else mprintf(" System file (%s) : %s (%d)\n",sys_time,
               sip->sys_file.FileName,sip->sys_file.size);

  if(!sip->config_file.exists)
  { mprintf(" No config file for %s.\n",name);
  }
  else mprintf(" Config file (%s) : %s (%d)\n",conf_time,
               sip->config_file.FileName,sip->config_file.size);

  if(!sip->config_file.exists || !sip->sys_file.exists) return(1);
 
  if(sip->differ)
  { if(sip->sys_file_older)
    { mprintf("   %s%s\n   %s\n",
              "The System file and the Config file differ ",
              "and the System file is the oldest.",
              "The system file needs to be updated."
             );
    }
    else
    { mprintf("   %s%s\n   %s\n",
              "The System file and the Config file differ ",
              "and the Config file is the oldest.",
              "The system file needs to be archived."
             );
    }
  }
  else
  { if(sip->sys_file_older)
    { mprintf("   %s%s\n",
              "The System file and the Config file are identical ",
              "but the System file is the oldest."
             );
    }
    else
    { mprintf("   %s\n",
              "The System file and the Config file are identical."
             );
    }
  }
  return(1);
}

/*****************************************************************/
/* Check that the config file and the system file are identical. */
/* Display a message only if the two file differ.                */
/* Increment the Check counter when the files are differents.    */
/* name : config name                                            */
/* sip : pointer to sysfile_file info structure                  */
/*****************************************************************/
int check(char *name,sysfile_info *sip,int echo)
{ char sys_time[E_LINE_SIZE],conf_time[E_LINE_SIZE];
 
  if(!sip->sys_file.exists)
  { if(echo) 
       mprintf("System file %s does not exist.\n",sip->sys_file.FileName);
    return(1);
  }
  if(!sip->config_file.exists)
  { mprintf("No config file for %s\n",name);
    return(1);
  }

  if(sip->differ)
  { strftime(sys_time,E_LINE_SIZE,"%H:%M:%S %d/%m/%Y",
           localtime(&sip->sys_file.time));
    strftime(conf_time,E_LINE_SIZE,"%H:%M:%S %d/%m/%Y",
           localtime(&sip->config_file.time));

    sysfile.CheckCounter++;
    mprintf("%s:\n",name);
    mprintf(" System file and Config file Differ:\n",name);
    mprintf(" System file (%s) : %s (%d)\n",sys_time,sip->sys_file.FileName,
                          sip->sys_file.size);
    mprintf(" Config file (%s) : %s (%d)\n",conf_time,
                          sip->config_file.FileName,sip->config_file.size);

    if(sip->sys_file_older)
    { mprintf(" The System file is the oldest.\n\n");
    }
    else
    { mprintf(" The Config file is the oldest.\n\n");
    }    
  }
  return(1);
}

/*******************************************************************/
/* List the config entries for which either the config file or the */
/* system file does not exist.                                     */
/* If neither of them exist, nothing is displayed                  */
/* name : config name                                              */
/* sip : pointer to sysfile_file info structure                    */
/*******************************************************************/
int is_not_conf(char *name,sysfile_info *sip,int echo)
{ 

  if(sip->config_file.exists && sip->sys_file.exists) return(1);
  
  if(!sip->config_file.exists && !sip->sys_file.exists)
  { return(1);
  }

  mprintf("%s :\n",name);

  if(!sip->sys_file.exists)
  { mprintf("System file %s does not exist.\n",sip->sys_file.FileName);
  }

  if(!sip->config_file.exists)
  { mprintf("No config file for %s\n",name);
  }

  return(1);
}

/******************************************************************/
/* Copy the config file into the system file.                     */
/* The config file must already exist.                            */
/* they are the config file for this config name.                 */
/* name : config name                                             */
/* sip : pointer to sysfile_file info structure                   */
/******************************************************************/
int install(char *name,sysfile_info *sip,int echo)
{ char Com[E_LONG_BUFF_SIZE];
  e_string *sp = 0;
  int err;

  if(!sip->config_file.exists)
  { mprintf("ERROR: The config file %s does not exist!\n",
             sip->config_file.FileName);
    return(1);
  }

  if(!sip->sys_file.exists)
  { mprintf("Creating the system file %s\n",sip->sys_file.FileName);
  }

  sprintf(Com,"/bin/cp %s %s",sip->config_file.FileName,
                                 sip->sys_file.FileName);

  if(echo) mprintf("%s\n",Com);

                    /* copy the config file into the system file */
  if(!(sp = e_exec_sh_command(Com,&err))) goto error;
  mprintf("%s\n",E_STRING_S(sp));

  if(sp) E_FREE(sp);
  return(1);

error:
  if(sp) E_FREE(sp);
  return(0);
}

/******************************************************************/
/* Copy the config file into the system file.                     */
/* The config file must already exist.                            */
/* Make a copy of the system file with the .DIST suffix if it     */
/* does not exists yet.                                           */
/* name : config name                                             */
/* sip : pointer to sysfile_file info structure                   */
/******************************************************************/
int first_install(char *name,sysfile_info *sip,int echo)
{ char Com[E_LONG_BUFF_SIZE];
  e_string *sp = 0;
  int err;

  if(!sip->config_file.exists)
  { mprintf("ERROR: The config file %s does not exist!\n",
            sip->config_file.FileName);
    return(1);
  }

  if(!sip->sys_file.exists)
  { mprintf("Creating the system file %s\n",sip->sys_file.FileName);
  }
  else 
  { /* create the file CONFIG.DIST if it does not exist */
    E_NULL_ERROR(sp = e_string_new(0,sip->sys_file.FileName,E_PATH_SIZE));
    E_NULL_ERROR(sp = e_string_append_buffer(sp,".DIST")); 
  
    /* if system.DIST does not exists, create it */
    if(!e_stat_file(E_STRING_S(sp)))
    { sprintf(Com,"/bin/cp %s %s",sip->sys_file.FileName,
                                   E_STRING_S(sp));

      mprintf("Creating the .DIST file %s\n",E_STRING_S(sp));
      E_FREE(sp);
                 /* copy the system file as system.DIST  */
      if(!(sp = e_exec_sh_command(Com,&err))) goto error;
      if(echo) mprintf("%s\n",E_STRING_S(sp));
      E_FREE(sp); 
      sp = 0;
    }
  }

  sprintf(Com,"/bin/cp %s %s",sip->config_file.FileName,
                                 sip->sys_file.FileName);

  if(echo) mprintf("%s\n",Com);

                    /* copy the config file into the system file */
  if(!(sp = e_exec_sh_command(Com,&err))) goto error;
  mprintf("%s\n",E_STRING_S(sp));

  if(sp) E_FREE(sp);
  return(1);

error:
  if(sp) E_FREE(sp);
  return(0);
}

/******************************************************************/
/* Copy the system file into the config file.                     */
/* The system file must exist.                                    */
/* The config file must already exist.                            */
/* Nothing happens if the config file is up to date.              */
/* Only .HOST config files are Archived.                          */
/* name : config name                                             */
/* sip : pointer to sysfile_file info structure                   */
/******************************************************************/
int archive(char *name,sysfile_info *sip,int echo)
{ char Com[E_LONG_BUFF_SIZE];
  e_string *sp = 0;
  int err;

  if(!sip->sys_file.exists)
  { mprintf("ERROR: The system file %s does not exist!\n",
            sip->sys_file.FileName);
    return(1);
  }

  if(sip->config_file.exists)
  { if(sip->differ) /* do not archive if there is no difference */
    {               /* .DEFAULT file : use -makedef command */
      if(sip->config_file.config_file_type & E_CONFIG_DEFAULT)
      { mprintf("Use the \"-makedef\" flag to update %s\n",
                 sip->config_file.FileName);
      }
              /* .SHARED file : use -makeshared command */
      if(sip->config_file.config_file_type & E_CONFIG_SHARED)
      { mprintf("Use the \"-makeshared\" flag to update %s\n",
                 sip->config_file.FileName);
      }
                    /* .GROUP file : use -makedef command */
      else if(sip->config_file.config_file_type & E_CONFIG_GROUP)
      { mprintf("Use the \"-makegroup\" flag to update %s\n",
                 sip->config_file.FileName);
      }
      else          /* Archive the file */ 
      { sprintf(Com,"/bin/cp %s %s",sip->sys_file.FileName,
                                       sip->config_file.FileName);

        if(echo) mprintf("%s\n",Com);

                 /* copy the system file into the config file */
        if(!(sp = e_exec_sh_command(Com,&err))) goto error;
        mprintf("%s\n",E_STRING_S(sp));
      }
    }
  }
  else             /* File does not exist, use \"-new\" flag instead */
  { mprintf(
     "Config file %s does not exists.\nUse the -new flag to update it.\n",
             sip->config_file.FileName);
  }


  if(sp) E_FREE(sp);
  return(1);
error:
  if(sp) E_FREE(sp);
  return(0);
}

/******************************************************************/
/* Copy the system file as a new config file : GROUP/CONFIG.HOSTS */
/* The config file must not exist yet.                            */
/* name : config name                                             */
/* sip : pointer to sysfile_file info structure                   */
/******************************************************************/
int new(char *name,sysfile_info *sip,int echo)
{ char Com[E_LONG_BUFF_SIZE];
  e_string *sp = 0;
  int err;

  if(!sip->sys_file.exists)
  { mprintf("ERROR: The system file %s does not exist!\n",
             sip->sys_file.FileName);
    return(1);
  }

  if(!sip->config_file.exists)
  { sprintf(Com,"/bin/cp %s %s",sip->sys_file.FileName,
                                 sip->config_file.FileName);
    if(echo) mprintf("%s\n",Com);

               /* copy the system file into the config file */
    if(!(sp = e_exec_sh_command(Com,&err))) goto error;
    mprintf("%s\n",E_STRING_S(sp));
  }
  else             /* File exists, use Archive instead */
  { mprintf("Config file %s exists.\nUse the Archive flag to update it.\n",
             sip->config_file.FileName);
  }

  if(sp) E_FREE(sp);
  return(1);

error:
  if(sp) E_FREE(sp);
  return(0);
  
}

/**************************************************************************/
/* Copy the system file as a specific config file:                        */
/* the DEFAULT config file : GROUP/CONFIG.DEFAULT                         */
/* the SHARED config file : GROUP/CONFIG.SHARED                           */
/* HOST_GROUP config file : GROUP/CONFIG.HOST_GROUP                       */
/* If the config file does not exist, it is created.                      */
/* name : config name                                                     */
/* sip : pointer to sysfile_file info structure                           */
/**************************************************************************/
int makedef(char *name,sysfile_info *sip,int echo)
{ char Com[E_LONG_BUFF_SIZE];
  e_string *sp = 0;
  int err;

  if(!sip->sys_file.exists)
  { mprintf("ERROR: The system file %s does not exist!\n",
            sip->sys_file.FileName);
    return(1);
  }

  sprintf(Com,"/bin/cp %s %s",sip->sys_file.FileName,
                                 sip->config_file.FileName);
  if(echo) mprintf("%s\n",Com);

               /* copy the system file into the config file */
  if(!(sp = e_exec_sh_command(Com,&err))) goto error;
  mprintf("%s\n",E_STRING_S(sp));

  if(sp) E_FREE(sp);
  return(1);

error:
  if(sp) E_FREE(sp);
  return(0);
}

/******************************************************************/
/* Display the differences between the config and the system file */
/* name : config name                                             */
/* sip : pointer to sysfile_file info structure                   */
/******************************************************************/
int diff(char *name,sysfile_info *sip,int echo)
{ char Com[E_LONG_BUFF_SIZE];
  e_string *sp = 0;
  int err;

  if(!sip->config_file.exists)
  { mprintf("No config file for %s.\n",name);
    return(1);
  }
  if(!sip->sys_file.exists)
  { mprintf("System file %s does not exist.\n",sip->sys_file.FileName);
    return(1);
  }

               /* Build the diff command   */
  sprintf(Com,"diff %s %s",sip->sys_file.FileName,
                            sip->config_file.FileName);

  if(echo) mprintf("%s\n",Com);

               /* Execute the diff command */
  if(!(sp = e_exec_sh_command(Com,&err))) goto error;

  if(E_STRING_S(sp)[0])
  { mprintf("%s:\n%s",name,E_STRING_S(sp));
  }

  if(sp) E_FREE(sp);
  return(1);

error:
  if(sp) E_FREE(sp);
  return(0);
}

/******************************************************************/
/* Display the differences between the config files of two hosts  */
/* name : config name                                             */
/* fip : file_info pointer for the host being tested.             */
/* host1 : name of first host (argument of diff_host)             */
/* sip : pointer to sysfile_file info structure                   */
/* host2 : name of second host (current host)                     */
/******************************************************************/
int diff_host(char *name,file_info *fip,char *host1,sysfile_info *sip,
              char *host2,int echo)
{ char Com[E_LONG_BUFF_SIZE];
  e_string *sp = 0;
  int err;

  if(!fip->exists || !sip->config_file.exists)
  { if(!fip->exists && !sip->config_file.exists)
    { if(echo) mprintf("No config file %s for %s and %s.\n",name,host1,host2);
    }
    else
    { if(!fip->exists)
      { mprintf("No config file %s for host %s.\n",name,host1);
      }
      if(!sip->config_file.exists)
      { mprintf("No config file %s for host %s.\n",name,host2);
      }
    }
    return(1);
  }

  if(!strcmp(fip->FileName,sip->sys_file.FileName))
  { /* same config file! */
    return(1);
  }
               /* Build the diff command   */
  sprintf(Com,"diff %s %s",fip->FileName,sip->sys_file.FileName);

  if(echo) mprintf("%s\n",Com);

               /* Execute the diff command */
  if(!(sp = e_exec_sh_command(Com,&err))) goto error;

  if(E_STRING_S(sp)[0])
  { mprintf("%s:\n%s",name,E_STRING_S(sp));
  }

  if(sp) E_FREE(sp);
  return(1);

error:
  if(sp) E_FREE(sp);
  return(0);
}

/******************************************************************/
/* Check the differences between the config files of two hosts    */
/* name : config name                                             */
/* fip : file_info pointer for the host being tested.             */
/* host1 : name of first host (argument of check_host)            */
/* sip : pointer to sysfile_file info structure                   */
/* host2 : name of second host (current host)                     */
/******************************************************************/
int check_host(char *name,file_info *fip,char *host1,sysfile_info *sip,
               char *host2,int echo)
{ char host1_time[E_LINE_SIZE],host2_time[E_LINE_SIZE];
  int test;
 
  if(!fip->exists || !sip->config_file.exists)
  { if(!fip->exists && !sip->config_file.exists)
    { if(echo) mprintf("No config file %s for %s and %s.\n",name,host1,host2);
    }
    else
    { if(!fip->exists)
      { if(echo) mprintf("No config file %s for host %s.\n",name,host1);
      }
      if(!sip->config_file.exists)
      { if(echo) mprintf("No config file %s for host %s.\n",name,host2);
      }
    }
    return(1);
  }

  if(!strcmp(fip->FileName,sip->sys_file.FileName))
  { /* same config file! */
    return(1);
  }

  if((test = files_are_equal(fip->FileName,sip->sys_file.FileName))<0) 
  { mprintf("ERROR reading %s or %s.\n",fip->FileName,sip->sys_file.FileName);
    return(0);
  }

  if(!test) /* file differ */
  { strftime(host1_time,E_LINE_SIZE,"%H:%M:%S %d/%m/%Y",
             localtime(&fip->time));
    strftime(host2_time,E_LINE_SIZE,"%H:%M:%S %d/%m/%Y",
             localtime(&sip->config_file.time));

    sysfile.CheckCounter++;
    mprintf("%s:\n",name);
    mprintf(" Config files Differ:\n",name);
    mprintf(" %s Config file (%s) : %s (%d)\n",host1,host1_time,fip->FileName,
                             fip->size);
    mprintf(" %s Config file (%s) : %s (%d)\n",host2,host2_time,
                             sip->config_file.FileName,sip->config_file.size);   
  }

  return(1);

}

/*****************************************************************/
/* Display the config file                                       */
/* name : config name                                            */
/* sip : pointer to sysfile_file info structure                  */
/*****************************************************************/
int cat_config(char *name,sysfile_info *sip,int echo)
{ FILE *fpt;
  char Buff[E_LONG_BUFF_SIZE];

  if(!sip->config_file.exists)
  { mprintf("No config file for %s.\n",name);
    return(1);
  }

  if(!(fpt = fopen(sip->config_file.FileName,"r")))
  { mprintf("Unable to open config file %s!",sip->config_file.FileName);
    return(1);
  }

  while(fgets(Buff,E_LONG_BUFF_SIZE,fpt))
  { mprintf("%s",Buff);
  }
  mprintf("\n",Buff);

  fclose(fpt);

  return(1);
}

/*****************************************************************/
/* Display the system file                                       */
/* name : config name                                            */
/* sip : pointer to sysfile_file info structure                  */
/*****************************************************************/
int cat_system(char *name,sysfile_info *sip,int echo)
{ FILE *fpt;
  char Buff[E_LONG_BUFF_SIZE];

  if(!sip->sys_file.exists)
  { mprintf("ERROR: System file does not exist.\n");
    return(1);
  }

  if(!(fpt = fopen(sip->sys_file.FileName,"r")))
  { mprintf("Unable to open system file %s!",sip->sys_file.FileName);
    return(1);
  }

  while(fgets(Buff,E_LONG_BUFF_SIZE,fpt))
  { mprintf("%s",Buff);
  }
  mprintf("\n",Buff);

  fclose(fpt);

  return(1);
}

/***********************************************************************/

/***********************************************/
/* Get the path name for the system file.      */
/* Get the last modification time and file size*/
/* Check that it is a file, not a directory.   */
/* Return the result in fp.                    */
/* Return : 1 : file found                     */
/*          0 : system file does not exist.    */
/* Link conf_data : 0                          */
/***********************************************/
int get_system_file(char *conf_name,e_aset *conf_data,file_info *fp)
{ e_string *sp = 0;
  struct stat statp; 
  int i,j,c;
  char *s;

  E_NULL_ERROR(fp);

                     /* Get system file name */
  E_NULL_ERROR(sp = e_aset_get_value_by_index(conf_data,1));

                     /* Copy the system file name in fp */
  /*
  strncpy(fp->FileName,E_STRING_S(sp),E_PATH_SIZE);
  fp->FileName[E_PATH_SIZE] = 0;
  */

  /* Copy filename and substitute $H with the host name */
  i = 0;
  s = E_STRING_S(sp);
  while((c = *s++) && (i < (E_PATH_SIZE-1)))
  { if(c == '$') /* variable : $H => hostname */
    { c = *s++;
      if(c != 'H')
      { mprintf("ERROR: Invalid variable ($%c) in system file name: %s!",
                 c,E_STRING_S(sp));
        goto error;
      }  
      j = 0;    /* copy hostname */
      while((fp->FileName[i] = HostName[j++]) && (i < (E_PATH_SIZE-1)))
      { i++;
      }
    }
    else fp->FileName[i++] = c;
  }
  fp->FileName[i] = 0;

               /* If the file exists, it must be a file */
  if(!stat(fp->FileName,&statp))
  { 
                     /* Check that the system file is a file */
    if(!S_ISREG(statp.st_mode))
    { e_error(E_ERROR_BAD_ARGUMENT,E_ERROR_LEVEL_USER,
              "System file %s is a directory, not a file",fp->FileName);
      goto error;
    }
    fp->exists = 1; 
  }
  else fp->exists = 0; 

                     /* Get the modification time and size */
  fp->time = statp.st_mtime;
  fp->size = statp.st_size;

  if(sp) E_FREE(sp);

  return(1);

error :
  if(sp) E_FREE(sp);
  return(0);
}

/*****************************************************************/
/* Get the path name for the config file.                        */
/* Get the last modification time and file size                  */
/* Check that it is a file, not a directory.                     */
/* conf_name : config name                                       */
/* conf_data : aset of data for this config file                 */
/* *fp : file_info to fill in                                    */
/* host : the host name.                                         */
/* NoCheck : if non NULL, don't fail if the config file exists,  */
/*       just put the GROUP/CONFIG.HOST path in fp where         */
/*       host can be DEFAULT or a HOST_GROUP name                */
/*       Check if that file exists (for the new command)         */
/* Return the result in fp.                                      */
/* Return : 1 : file found                                       */
/*          0 : no config file for this host                     */
/* Link conf_data : 0                                            */
/*****************************************************************/
int get_config_file(char *conf_name,char *conf_dir,e_aset *conf_data,
                    file_info *fp,char *host,int NoCheck)
{ e_string *sp = 0,*config_name = 0,*group = 0;
  e_name *np = 0;
  struct stat statp; 
  int found = 0;

  if(!fp || !conf_data || !host) goto error;

  fp->exists = 0;

                     /* Get config file group */
  E_NULL_ERROR(group = e_aset_get_value_by_index(conf_data,0));

                /* sp : config file name :          */
                /* ConfigDir/Group/Config.EXTENSION */

  E_NULL_ERROR(sp = e_string_new(0,conf_dir,E_PATH_SIZE));
  E_NULL_ERROR(sp = e_string_append_buffer(sp,"/"));
  E_NULL_ERROR(sp = e_string_append_buffer(sp,E_STRING_S(group)));
  E_NULL_ERROR(sp = e_string_append_buffer(sp,"/"));
  E_NULL_ERROR(sp = e_string_append_buffer(sp,conf_name));
  E_NULL_ERROR(sp = e_string_append_buffer(sp,"."));

  E_NULL_ERROR(config_name = e_string_append_buffer(E_USE(sp),host));
 
  if(NoCheck)
  {  strncpy(fp->FileName,E_STRING_S(config_name),E_PATH_SIZE);
     E_FREE(config_name); config_name = 0;

     fp->config_file_type = E_CONFIG_HOST; /* by default */
     if(!strcmp(host,"DEFAULT")) fp->config_file_type = E_CONFIG_DEFAULT;
     if(!strcmp(host,"SHARED"))  fp->config_file_type = E_CONFIG_SHARED;

     if(stat(fp->FileName,&statp)) fp->exists = 0;
     else 
     { fp->exists = 1;
       fp->time = statp.st_mtime;
       fp->size = statp.st_size;
     }
     goto end;
  }


  fp->config_file_type = E_CONFIG_HOST; /* This is a .HOST config file */

  if(stat(E_STRING_S(config_name),&statp))
  { E_FREE(config_name); config_name = 0;
 
                /* config.HOST does not exist, try each group in turn */   
    e_aset_next(sysfile.this_host_groups,E_NEXT_RESET,0);
  
    fp->config_file_type = E_CONFIG_GROUP; /* This is a .GROUP config file */

    while(!found && e_aset_next(sysfile.this_host_groups,E_NEXT_GET_NEXT,
                                (void **)&np)>0)
    { 
                                           /* does config.GROUP exist? */   
      E_NULL_ERROR(config_name = e_string_append_buffer(E_USE(sp),
                                                      E_NAME_NAME(np)));

      if(!stat(E_STRING_S(config_name),&statp)) found = 1;
      else { E_FREE(config_name); config_name = 0; }
      if(np) E_FREE(np); 
      np = 0;
    }     

    if(!found)
    {                                    /* does config.DEFAULT exist? */ 
      E_NULL_ERROR(config_name = e_string_append_buffer(E_USE(sp),"DEFAULT"));
      if(stat(E_STRING_S(config_name),&statp)) 
      { e_error(E_ERROR_BAD_ARGUMENT,E_ERROR_LEVEL_USER,
                "sysf on %s: no config file for %s",host,conf_name);
        goto error;
      }
      fp->config_file_type = E_CONFIG_DEFAULT; /* A .DEFAULT config file */
    }
  }

  strncpy(fp->FileName,E_STRING_S(config_name),E_PATH_SIZE);
  E_FREE(config_name); config_name = 0;

                     /* Check that the config file is a file */
  if(!S_ISREG(statp.st_mode))
  { e_error(E_ERROR_BAD_ARGUMENT,E_ERROR_LEVEL_USER,
            "sysf :  configfile %s is a directory, not a file",fp->FileName);
    goto error;
  }

                     /* Get the modification time and size */
  fp->time = statp.st_mtime;
  fp->size = statp.st_size;
  fp->exists = 1;

end:
  if(sp) E_FREE(sp);
  if(group) E_FREE(group);

  return(1);

error :
  if(config_name) E_FREE(config_name);
  if(group) E_FREE(group);
  if(sp) E_FREE(sp);
  return(0);
}


/***********************************************/
/* Get the flags for the system file.          */
/* Return the result in *flag.                 */
/* Return : 1 : OK                             */
/*          0 : error                          */
/* Link conf_data : 0                          */
/***********************************************/
int get_system_file_flags(char *conf_name,e_aset *conf_data,int *flags)
{ e_string *flags_s = 0;
  int i,c;
  char *sp;

  if(!conf_name || !conf_data || !flags) goto error;

  *flags = 0;
                     /* Get the flags if there is one */
  if(!(flags_s = e_aset_get_value_by_index(conf_data,3))) goto end;

  sp = E_STRING_S(flags_s);
  i = 0;
  while((c = sp[i]))
  { switch(c) 
    { case 'C' : *flags |= E_SYSF_FLAG_C;
               break;
      case 'I' : *flags |= E_SYSF_FLAG_I;
               break;
      case 'U' : *flags |= E_SYSF_FLAG_U;
               break;
      case 's' : *flags |= E_SYSF_FLAG_s;
               break;
      case 'h' : *flags |= E_SYSF_FLAG_h;
               break;
      default :
          mprintf("Config file %s : invalid flag %c\n",conf_name,c);
          goto error;
    }
    i++;
  }

end:
  if(flags_s) E_FREE(flags_s);

  return(1);

error :
  if(flags_s) E_FREE(flags_s);
  return(0);
}

/**************************************************************/
/* Get the path name for the system command if it exists.     */
/* Check that it is a file, not a directory.                  */
/* Return the result in fp.                                   */
/*         fp->sys_com.exists is 0 if command does not exist. */
/* Return : 1 : OK                                            */
/*          0 : error.                                        */
/* Link conf_data : 0                                         */
/**************************************************************/
int get_system_command(char *conf_name,char *conf_dir,e_aset *conf_data,
                       file_info *fp)
{ e_string *sp = 0,*sysname = 0;
  struct stat statp; 

  if(!conf_name || !conf_data || !fp) goto error;

                     /* Get system file name */
  E_NULL_ERROR(sysname = e_aset_get_value_by_index(conf_data,0));

                     /* Copy the system command name in fp */
  E_NULL_ERROR(sp = e_string_new(0,conf_dir,E_PATH_SIZE));
  E_NULL_ERROR(sp = e_string_append_buffer(sp,"/BIN/"));
  E_NULL_ERROR(sp = e_string_append_buffer(sp,conf_name));

  strncpy(fp->FileName,E_STRING_S(sp),E_PATH_SIZE);
  fp->FileName[E_PATH_SIZE] = 0;

               /* If the file exists, it must be a file */
  if(!stat(fp->FileName,&statp))
  { 
                     /* Check that the system command is a file */
    if(!S_ISREG(statp.st_mode))
    { e_error(E_ERROR_BAD_ARGUMENT,E_ERROR_LEVEL_USER,
              "System command %s is a directory, not a file",fp->FileName);
      goto error;
    }

                     /* Check that the system command is executable */
    if(!((statp.st_mode & S_IXUSR) && (statp.st_mode & S_IXGRP)))
    { e_error(E_ERROR_BAD_ARGUMENT,E_ERROR_LEVEL_USER,
              "System command %s is not executable",fp->FileName);
      goto error;
    }
    fp->exists = 1; 
  }
  else fp->exists = 0; 

                     /* Get the modification time and size */
  fp->time = statp.st_mtime;
  fp->size = statp.st_size;

  if(sp) E_FREE(sp);
  if(sysname) E_FREE(sysname);

  return(1);

error :
  if(sp) E_FREE(sp);
  if(sysname) E_FREE(sysname);
  return(0);
}

/*************************************************/
/* Execute the system command file if it exists. */
/* SYS_COM ACTION COM SYSFILE_NAME CONFIG_FILE   */
/* The output is either displayed or emailed.    */
/* action : one of                               */
/*    -CreateSysFile                             */
/*    -UpdateSysFile                             */
/*    -PreInstall                                */
/*    -PostInstall                               */
/* command : sysf command (-install, ...)        */
/* fp : file info for this system file.          */
/* Return 1 : OK                                 */
/*        0 : error                              */
/*************************************************/
int execute_sys_com(char *action, char *command,sysfile_info *sip,int echo)
{ e_string *esp = 0;
  char *argv[7],*echo_s;

  if(!action || !command || !sip)
  { goto error;
  }

        /* No config file : no point to do anything */
  if(!sip->config_file.FileName[0])
  { return(1);
  }

  if(echo) echo_s = "on";
  else     echo_s = "off";

  if(!sip->sys_com.exists)
  { e_error(E_ERROR_BAD_ARGUMENT,E_ERROR_LEVEL_USER,
              "Command %s does not exist!",sip->sys_com.FileName);
    goto error;
  }
 
  argv[0] = sip->sys_com.FileName;
  argv[1] = action;
  argv[2] = command;
  argv[3] = sip->sys_file.FileName;
  argv[4] = sip->config_file.FileName;
  argv[5] = echo_s;
  argv[6] = 0;

  if((esp = e_exec_pipe_read(argv)))
  { mprintf("%s",E_STRING_S(esp));
    E_FREE(esp);
  }
  return(1);
  
error:
  if(esp) E_FREE(esp);
  return(0);
}

/********************************************************************/
/* Test if the sysfile and config file are identical and up to date */
/* sip : pointer to sysfile_file info structure                     */
/* Return : 1 sysfile and config file are identical                 */
/*          0 sysfile and config file differ                        */
/********************************************************************/
int test_sysfile(sysfile_info *sip)
{ 
  sip->differ = 0;
  sip->sys_file_older = 0;
  if (sip->sys_file.size != sip->config_file.size)
  { sip->differ = 1;
  }
  else
  { if(!files_are_equal(sip->sys_file.FileName,sip->config_file.FileName))
     sip->differ = 1;
  }

  if(sip->sys_file.time < sip->config_file.time) sip->sys_file_older = 1;

  return(!sip->differ);
}

/*************************************************/
/* Test if the content of two files is identical */
/* Return : 1 : file are identical               */
/*        : 0 : files differ                     */
/*         -1 : error                            */
/*************************************************/
int files_are_equal(char *f1,char *f2)
{ int s1,s2,equal;
  FILE *fpt1,*fpt2;
  char Buff1[E_LONG_BUFF_SIZE],Buff2[E_LONG_BUFF_SIZE];

  if(!f1 || !f2) goto error;
  if(!(fpt1=fopen(f1,"r"))) goto error;
  if(!(fpt2=fopen(f2,"r"))) { fclose(fpt1); goto error;}
  
  equal = 1;
  do
  { s1 = fread(Buff1,1,E_LONG_BUFF_SIZE,fpt1);
    s2 = fread(Buff2,1,E_LONG_BUFF_SIZE,fpt2);
             /* files differ if the blocks are different */

    if(s1 != s2) equal = 0;
    else if(memcmp(Buff1,Buff2,s1)) equal = 0;
  }
  while(s1 && equal);

  fclose(fpt1);
  fclose(fpt2);
  return(equal);

error: 
  
  return(-1);
}




