#ifndef SYSF_COM_H
#define SYSF_COM_H
#include <time.h>

/* set of flags */
#define E_SYSF_FLAG_C 1
#define E_SYSF_FLAG_I 2
#define E_SYSF_FLAG_U 4
#define E_SYSF_FLAG_s 8
#define E_SYSF_FLAG_h 16

#define E_CONFIG_HOST    1
#define E_CONFIG_GROUP   2
#define E_CONFIG_DEFAULT 4
#define E_CONFIG_SHARED  8

typedef struct {
  char FileName[E_PATH_SIZE];  /* File name              */
  time_t time;                 /* Last modification time */
  int size;                    /* File size              */
  int exists;                  /* 1 if the file exists   */
  int config_file_type;        /* HOST, GROUP or DEFAULT */
} file_info;

typedef struct {
  file_info sys_file;
  file_info config_file;
  file_info sys_com;
  int differ;
  int sys_file_older;
  int flags;
} sysfile_info;

int get_info(char *name,sysfile_info *sip,int echo);
int check(char *name,sysfile_info *sip,int echo);
int is_not_conf(char *name,sysfile_info *sip,int echo);
int install(char *name,sysfile_info *sip,int echo);
int first_install(char *name,sysfile_info *sip,int echo);
int archive(char *name,sysfile_info *sip,int echo);
int new(char *name,sysfile_info *sip,int echo);
int makedef(char *name,sysfile_info *sip,int echo);
int diff(char *name,sysfile_info *sip,int echo);
int diff_host(char *name,file_info *fip,char *host1,sysfile_info *sip,
              char *host2,int echo);
int check_host(char *name,file_info *fip,char *host1,sysfile_info *sip,
               char *host2,int echo);
int cat_config(char *name,sysfile_info *sip,int echo);
int cat_system(char *name,sysfile_info *sip,int echo);
int get_system_file(char *conf_name,e_aset *conf_data,file_info *fp);
int get_config_file(char *conf_name,char *conf_dir,e_aset *conf_data,
                    file_info *fp,char *host,int NoCheck);
int get_system_file_flags(char *conf_name,e_aset *conf_data,int *flags);
int get_system_command(char *conf_name,char *conf_dir,e_aset *conf_data,
                    file_info *fp);
int execute_sys_com(char *action, char *command,sysfile_info *sip,int echo);
int test_sysfile(sysfile_info *sip);
int files_are_equal(char *f1,char *f2);
#endif
