#include "sysf.h"
#include "sysf_init.h"
#include <stdio.h>
#include <ctype.h>
#include <stdarg.h>

/***************************************************************/
/* Read the host.conf file into the list sysfile.hosts_groups. */
/* hosts_groups = { Group1 = {Host1 Host2 ... };               */
/*                  Group2 = {Host3 Host4 ... };               */
/* }                                                           */
/* The host names Hosti are strings                            */
/* Return : 1 : OK                                             */
/*          0 : error                                          */
/***************************************************************/
int read_sysf_hosts(char *config_dir,char *hosts_file)
{ e_tokeniser *tp = 0;
  e_aset *groups = 0,*grp = 0,*asp;
  e_name *group_name = 0;
  int state;
  e_flag mode = E_TOKENISER_MODE_HASH_COMMENT;
  e_string *file_path=0;

                  /* create host_file path name */
  E_NULL_ERROR(file_path = e_string_new(0,config_dir,E_PATH_SIZE));
  E_NULL_ERROR(file_path = e_string_append_buffer(file_path,"/")); 
  E_NULL_ERROR(file_path = e_string_append_buffer(file_path,hosts_file)); 

  if(!(tp = e_tokeniser_new(0,e_charstream_new(0,
                                E_CHARSTREAM_TYPE_FILE,
                                e_file_new(0,0,E_STRING_S(file_path),"r"),
                                E_MODE_READ),
                            mode,E_SYSF_DEF_WORD,E_SYSF_DEF_OP,
                            E_SYSF_DEF_PUNCT,E_SYSF_DEF_SPACES)))
  { goto error;
  }

  E_NULL_ERROR(groups = e_aset_new(0)); 

  state = STATE_NEW;
  while(e_tokeniser_next(tp) > 0)
  { switch(tp->type)
    { 
      case E_TOKENISER_WORD_TOKEN :
            if(state == STATE_NEW)         /* new groups */
            { if(!isupper(E_TOKENISER_BUFF(tp)[0]))
              { e_error_syntax("sysf: Syntax error in hosts.conf, line %d: \n      Group name (%s) must start with an upper case letter!\n",
                E_TOKENISER_LINE_NO(tp),E_TOKENISER_BUFF(tp));
                goto error;
	      }
              E_NULL_ERROR((group_name = e_name_new(E_TOKENISER_BUFF(tp))));

              state = STATE_EQUAL;
              E_NULL_ERROR(grp = e_aset_new(0)); 

	    }          
            else if( state == STATE_EQUAL) /* Error  */
            { goto error1;
	    }          
            else                         /* add 1 host to the group */
	    { if(isupper(E_TOKENISER_BUFF(tp)[0]))
	      {                          /* insert group */
                E_NULL_ERROR(asp = (e_aset *)e_aset_get_value_by_key(groups,
                                         e_name_new(E_TOKENISER_BUFF(tp)),0));
                E_NULL_ERROR(grp = e_aset_append(grp,asp)); asp = 0;
	      }
              else
              { E_NULL_ERROR((grp = e_aset_append_item(grp,0,
                               e_string_new(0,E_TOKENISER_BUFF(tp),0))));
	      }
	    }        
            break;

      case E_TOKENISER_OP_TOKEN :       /* read the = op */
            if((E_TOKENISER_BUFF(tp)[0] != '=') || (state != STATE_EQUAL))
               goto error1;

            state = STATE_HOST;
            break;

      case E_TOKENISER_PUNCT_TOKEN :       /* end of host list */
            if ((E_TOKENISER_BUFF(tp)[0] != ';') || (state != STATE_HOST))
               goto error1;
            
	                          /* Append host list to the group list */
            groups = e_aset_append_item(groups,group_name,grp);
            grp = 0; group_name = 0;

            state = STATE_NEW;

            break;
    }
  }

  if(tp) E_FREE(tp);
  if(file_path) E_FREE(file_path);

  sysfile.hosts_groups = groups;
  return(1);

error1:
   e_error_syntax(
   "sysf: Syntax error reading hosts.conf file line %d: unexpected char %c\n", 
                  E_TOKENISER_LINE_NO(tp),E_TOKENISER_BUFF(tp)[0]);
error:
  if(tp)         E_FREE(tp);
  if(grp)        E_FREE(grp);
  if(group_name) E_FREE(group_name);
  if(file_path)  E_FREE(file_path);
  return(0);
}

/*****************************************************************************/
/* Read the sysf.conf file into the list sysfile.configs.                    */
/* configs={ Conf1 = { Group1 System_file1 Config_file1 [Flag1]};            */
/*           Conf2 = { Group2 System_file2 Config_file2 [Flag2]};            */
/* }                                                                         */
/* Conf1 are sets of 3, 4 or 5 strings                                       */
/* Return : 1 : OK                                                           */
/*          0 : error                                                        */
/*****************************************************************************/
int read_sysf_config(char *config_dir,char *config_file)
{ e_tokeniser *tp = 0;
  e_aset *configs = 0,*item_list = 0;
  e_name *item_name = 0;
  int state;
  e_flag mode = E_TOKENISER_MODE_HASH_COMMENT;
  e_string *file_path=0;

                  /* create host_file path name */
  E_NULL_ERROR(file_path = e_string_new(0,config_dir,E_PATH_SIZE));
  E_NULL_ERROR(file_path = e_string_append_buffer(file_path,"/")); 
  E_NULL_ERROR(file_path = e_string_append_buffer(file_path,config_file)); 

  if(!(tp = e_tokeniser_new(0,e_charstream_new(0,
                                 E_CHARSTREAM_TYPE_FILE,
                                 e_file_new(0,0,E_STRING_S(file_path),"r"),
                                 E_MODE_READ),
                            mode,E_SYSF_DEF_WORD,E_SYSF_DEF_OP,
                            E_SYSF_DEF_PUNCT,E_SYSF_DEF_SPACES)))
  { goto error;
  }

  E_NULL_ERROR(configs = e_aset_new(0)); 

  state = STATE_NEW;
  while(e_tokeniser_next(tp) > 0)
  { switch(tp->type)
    { 
      case E_TOKENISER_WORD_TOKEN :
            if(state == STATE_NEW)         /* new configs */
            { E_NULL_ERROR((item_name = e_name_new(E_TOKENISER_BUFF(tp))));

              state = STATE_EQUAL;
              E_NULL_ERROR(item_list = e_aset_new(0)); 

	    }          
            else if( state == STATE_EQUAL) /* Error  */
            { goto error1;
	    }          
            else                         /* add 1 the config item */
	    { E_NULL_ERROR((item_list = e_aset_append_item(item_list,0,
                               e_string_new(0,E_TOKENISER_BUFF(tp),0))));
	    }        
            break;

      case E_TOKENISER_OP_TOKEN :       /* read the = op */
           if((E_TOKENISER_BUFF(tp)[0] != '=') || (state != STATE_EQUAL))
               goto error1;

            state = STATE_ITEM;
            break;

      case E_TOKENISER_PUNCT_TOKEN :       /* end of host list */
            if ((E_TOKENISER_BUFF(tp)[0] != ';') || (state != STATE_ITEM))
               goto error1;
            if(E_ASET_CARDINAL(item_list) < 3)
	    { e_error_syntax(
       "sysf: Syntax error reading sysf.conf file line %d:\n      To few parameter for item %s\n",
                  E_TOKENISER_LINE_NO(tp),E_NAME_NAME(item_name));
              E_FREE(item_name);
              goto error;
	    }            

	                          /* Append item list to the config list */
            configs = e_aset_append_item(configs,item_name,item_list);
            item_list = 0; item_name = 0;

            state = STATE_NEW;

            break;
    }
  }

  if(tp) E_FREE(tp);
  if(file_path) E_FREE(file_path);
  sysfile.configs = configs;
  return(1);

error1:
   e_error_syntax(
    "sysf: Syntax error reading sysf.conf file line %d: unexpected char %c\n", 
                  E_TOKENISER_LINE_NO(tp),E_TOKENISER_BUFF(tp)[0]);
error:
  if(tp)        E_FREE(tp);
  if(item_list) E_FREE(item_list);
  if(item_name) E_FREE(item_name);
  if(file_path) E_FREE(file_path);
  return(0);
}

/***********************************************************/
/* Create a set with all the hosts selected in hosts list: */
/* a) Split the list into an aset.                         */
/* b) Resolve each group name                              */ 
/* Return : aset of host names.                            */
/*          0 : error                                      */
/* Link : Return +1                                        */
/***********************************************************/
e_aset *read_args_hosts(char *hosts_list)
{ e_aset *hosts = 0,*asp = 0,*asp2;
  e_string *sp;

  E_NULL_ERROR(asp = e_aset_split(e_string_new(0,hosts_list,0),0,","));
  E_NULL_ERROR(hosts = e_aset_new(0));

  e_aset_next(asp,E_NEXT_RESET,0);

  while(e_aset_next(asp,E_NEXT_GET_NEXT,(void **)&sp) > 0)
  { if(isupper(E_STRING_S(sp)[0]))
    {               /* Group : append list of hosts for that group */
      if(!(asp2 = (e_aset *)e_aset_get_value_by_key(sysfile.hosts_groups,
                                         e_name_new(E_STRING_S(sp)),0)))
      { e_reset();
        e_error_syntax(
    "sysf : invalid argument, Host Group \"%s\" not defined!",E_STRING_S(sp));
        goto error;
      }
      E_NULL_ERROR(hosts = e_aset_append(hosts,asp2)); 
    }
    else            /* 1 host : append the host name */
    { hosts = e_aset_append_item(hosts,0,sp);
      sp = 0;
      E_NULL_ERROR(hosts);
    }
    if(sp) E_FREE(sp);
  }
  
  return(hosts);

error:
  if(asp) E_FREE(asp);
  if(hosts) E_FREE(hosts);
  return(0); 
 
}

/********************************************************************/
/* Build a proprety list of config groups (sysfile.configs).        */
/* The keys are the name of the groups and the value are the set of */
/* all the config file (as e_name) belonging to that group.         */
/* The set is saved in sysfile.config_groups.                       */
/* config_groups = { Group1 = { conf1 conf2 ... };                  */
/*                   Group2 = { conf3 conf3 ... };                  */
/* }                                                                */
/* Return : 1 OK                                                    */
/*          0 Error                                                 */
/********************************************************************/
int make_config_group_lists()
{ e_name *np,*group_name;
  e_aset *asp = 0,*config_groups = 0;
  e_string *group;

  e_aset_next(sysfile.configs,E_NEXT_RESET,0);
  
  E_NULL_ERROR(config_groups = e_aset_new(0));

             /* Scan the list of all config files         */
             /* Build the property list :                 */
             /* { grp1 = { conf1 conf2 ..}; grp1 = { ..}} */
  while(e_aset_next(sysfile.configs,E_NEXT_GET_CUR_KEY,(void **)&np) > 0)
  { e_aset_next(sysfile.configs,E_NEXT_GET_NEXT,(void **)&asp);

             /* The first item is the group name          */
    E_NULL_ERROR(group = e_aset_get_value_by_index(asp,0));

             /* Convert it to a  name                     */
    group_name = e_name_new(E_STRING_S(group));

             /* Property values are the config names      */
    E_NULL_ERROR(e_aset_add_property(config_groups,group_name,np,0));
  }

  sysfile.config_groups = config_groups;
  return(1);

error:
  if(asp) E_FREE(asp);
  if(config_groups) E_FREE(config_groups);
  return(0);
}


/****************************************************************/
/* Create an aset with all the host group this host belongs to  */
/* and save it in sysfile.this_host_groups.                     */
/* The aset returned is a set of e_names.                       */
/* They are in the order in which the groups were defined.      */
/* Read the hosts groups from sysfile.hosts_groups              */
/* Return : 1 : OK                                              */
/*          0 : error                                           */
/* Link : host : 0                                              */
/*        sysfile.this_host_groups : +1                         */
/****************************************************************/
int get_this_host_groups(e_string *host)
{ e_aset *asp = 0,*h_grp = 0;
  e_name *np = 0;

  sysfile.this_host_groups = 0;
  E_NULL_ERROR(asp = e_aset_new(0));

  e_aset_next(sysfile.hosts_groups,E_NEXT_RESET,0);
  
             /* Scan all the host groups                  */
  while(e_aset_next(sysfile.hosts_groups,E_NEXT_GET_CUR_KEY,(void **)&np)>0)
  { e_aset_next(sysfile.hosts_groups,E_NEXT_GET_NEXT,(void **)&h_grp);

    if(e_aset_get_element_index(h_grp,E_USE(host),0) >= 0)
    {                    /* host belong to this group                   */
                         /* add the group name to the set as an e_name. */
      E_NULL_ERROR(asp = e_aset_append_item(asp,0,E_USE(np)));
    }
    if(np) E_FREE(np); 
    np = 0;
  } 
  sysfile.this_host_groups = asp;

  return(1);

error:
  if(h_grp) E_FREE(h_grp);
  if(asp) E_FREE(asp);
  if(np) E_FREE(np);
  return(0);
}

/************************* MAIL *************************/
/**************************************************/
/* Print message in a string or on the screen    */
/* sysfile.message is null : print on the screen */
/* otherwise append to sysfile.message           */
/*************************************************/
void mprintf(const char *fmt, ...)
{ va_list args;
  char buff[E_LONG_BUFF_SIZE];

  va_start(args, fmt);
  if(sysfile.messages)
  { vsprintf(buff,fmt, args); 
    sysfile.messages = e_string_append_buffer(sysfile.messages,buff);
  }
  else
  { vprintf(fmt, args); 
  }
  va_end(args);
}

/*************************************************/
/* Email sysfile.message to address              */
/* Return : 1 : OK                               */
/*          0 : error                            */
/*************************************************/
int mail(char *address,char *subject)
{ FILE *pfpt;
  e_string *com;

  com = e_string_new(0,MAIL_COM,E_LONG_BUFF_SIZE);

  com = e_string_append_buffer(com," -s \"");
  com = e_string_append_buffer(com,subject);
  com = e_string_append_buffer(com,"\" ");
  com = e_string_append_buffer(com,address);

  if(!(pfpt = (FILE *)popen(E_STRING_S(com),"w")))
  { mprintf("Error executing the command: %s\n",E_STRING_S(com));
    return(0);
  } 

  if(sysfile.messages) fputs(E_STRING_S(sysfile.messages),pfpt);

  pclose(pfpt);

  return(1);
}

/************************* TESTS *************************/

/*****************************/
/* Display the config groups */
/*****************************/
void display_config_groups()
{ e_name *np;

  e_aset_next(sysfile.config_groups,E_NEXT_RESET,0);

  while(e_aset_next(sysfile.config_groups,E_NEXT_GET_NEXT_KEY,(void **)&np)>0)
  { mprintf("%s ",E_NAME_NAME(np));
    E_FREE(np);
  }
  mprintf("\n");

}

/*************************************************/
/* Display all the config files by config groups */
/*************************************************/
void display_all_config_files()
{ e_string *sp;
  e_aset *asp;  
  e_name *np;

  e_aset_next(sysfile.config_groups,E_NEXT_RESET,0);

  while(e_aset_next(sysfile.config_groups,E_NEXT_GET_CUR_KEY,(void **)&np) > 0)
  { mprintf("%s =",E_NAME_NAME(np));

    if(e_aset_next(sysfile.config_groups,E_NEXT_GET_NEXT,(void **)&asp) > 0) 

    e_aset_next(asp,E_NEXT_RESET,0);
    while(e_aset_next(asp,E_NEXT_GET_NEXT,(void **)&sp) > 0)
    { mprintf(" %s",E_STRING_S(sp));
      E_FREE(sp);
    } 
    mprintf(";\n");
    E_FREE(asp);
  }
}

/*************************************************/
/* Display all the config files parameters       */
/*************************************************/
void display_all_configs()
{ e_string *sp;
  e_aset *asp;  
  e_name *np;

  e_aset_next(sysfile.configs,E_NEXT_RESET,0);

  while(e_aset_next(sysfile.configs,E_NEXT_GET_CUR_KEY,(void **)&np) > 0)
  { mprintf("%s =",E_NAME_NAME(np));

    if(e_aset_next(sysfile.configs,E_NEXT_GET_NEXT,(void **)&asp) > 0) 

    e_aset_next(asp,E_NEXT_RESET,0);
    while(e_aset_next(asp,E_NEXT_GET_NEXT,(void **)&sp) > 0)
    { mprintf(" %s",E_STRING_S(sp));
      E_FREE(sp);
    } 
    mprintf(";\n");
    E_FREE(asp);
  }
}

/*************************************************/
/* Display all the config files by config groups */
/* Return : 1 : OK                               */
/*          0 : error                            */
/*************************************************/
int display_one_group(char *group)
{ e_string *sp;
  e_aset *asp = 0;  
  e_name *np;

  E_NULL_ERROR(np = e_name_new(group));

  asp = e_aset_get_value_by_key(sysfile.config_groups,np,0);

  if(!asp)
  { mprintf("sysf : Unknown group : %s\n",group);
    goto error;
  }

  mprintf("%s =",group);

  e_aset_next(asp,E_NEXT_RESET,0);
  while(e_aset_next(asp,E_NEXT_GET_NEXT,(void **)&sp) > 0)
  { mprintf(" %s",E_STRING_S(sp));
    E_FREE(sp);
  } 
  mprintf(";\n");
  E_FREE(asp);
  
  return(1);

error:
  if(asp) E_FREE(asp);
  return(0);
}

/************************************/
/* Display the lists of host groups */
/************************************/
void display_hosts_groups()
{ e_string *sp;
  e_aset *asp;  
  e_name *np;

  e_aset_next(sysfile.hosts_groups,E_NEXT_RESET,0);

  while(e_aset_next(sysfile.hosts_groups,E_NEXT_GET_CUR_KEY,(void **)&np) > 0)
  { mprintf("%s =",E_NAME_NAME(np));

    if(e_aset_next(sysfile.hosts_groups,E_NEXT_GET_NEXT,(void **)&asp) > 0) 

    e_aset_next(asp,E_NEXT_RESET,0);
    while(e_aset_next(asp,E_NEXT_GET_NEXT,(void **)&sp) > 0)
    { mprintf(" %s",E_STRING_S(sp));
      E_FREE(sp);
    } 
    mprintf(";\n");
    E_FREE(asp);
  }
}

/************************************/
/* Display the lists of host groups */
/************************************/
void display_host_group(char *group)
{ e_string *sp;
  e_aset *asp;  
  e_name *np;

  e_aset_next(sysfile.hosts_groups,E_NEXT_RESET,0);

  while(e_aset_next(sysfile.hosts_groups,E_NEXT_GET_CUR_KEY,(void **)&np) > 0)
  { if(e_aset_next(sysfile.hosts_groups,E_NEXT_GET_NEXT,(void **)&asp) > 0) 

    if(!strcmp(group,E_NAME_NAME(np)))
    { e_aset_next(asp,E_NEXT_RESET,0);
      while(e_aset_next(asp,E_NEXT_GET_NEXT,(void **)&sp) > 0)
      { mprintf("%s ",E_STRING_S(sp));
        E_FREE(sp);
      } 
      E_FREE(asp);
    }
  }
  mprintf("\n");
}

void display_configs()
{ e_string *sp;

  if((sp = E_TO_STRING(sysfile.configs,0,0)))
  { mprintf("%s\n",E_STRING_S(sp));
    E_FREE(sp);
  }
}
