#ifndef SYSF_INIT_H
#define SYSF_INIT_H

#include <easy.h>

#define E_SYSF_DEF_WORD   "_@$a-zA-Z0-9--./"
#define E_SYSF_DEF_OP     "!#%&*+<=>\\\\^?|~"
#define E_SYSF_DEF_PUNCT  "\"'(),:;`[]{}"
#define E_SYSF_DEF_SPACES " \a\b\t\v\f\r"

#define STATE_NEW   1
#define STATE_EQUAL 2
#define STATE_HOST  3
#define STATE_ITEM  3

/*****************************************************************************/
/* hosts_groups : list of all the host groups:                               */
/* hosts_groups = { h_group1 = {Host1 Host2 ... };                           */
/*                  h_group2 = {Host3 Host4 ... };                           */
/* }                                                                         */
/* config_groups : list of all config groups:                                */
/* config_groups = { Group1 = { conf1 conf2 ... };                           */
/*                   Group2 = { conf3 conf4 ... };                           */
/* }                                                                         */
/* where conf1 conf2 ...  are e_names.                                       */
/* configs : ist of all the config files with their properties:              */
/* configs={ Conf1 = { Group1 System_file1 Config_file1 [PreCom [PostCom]]}; */
/*           Conf2 = { Group2 System_file2 Config_file2 [PreCom [PostCom]]}; */
/* }                                                                         */
/* hosts : list of selected hosts.                                           */
/* hosts = { Host1 Host2 ... };                                              */
/* arg_configs : list of selected sys file.                                  */
/* arg_configs = { conf1 conf2 ... }                                         */
/* where conf1, conf2 ... are  names.                                        */
/* this_host_groups : list of groups this host belongs to.                   */
/* hosts = { h_group1 h_group2 ... };                                        */
/* where h_group1, h_group2 ... are e_names.                                 */
/*****************************************************************************/
struct sysf_par {
  char name[E_BUFF_SIZE];              /* config name      */
  char config_root[E_PATH_SIZE];       /* config directory */
  char system_file[E_PATH_SIZE];       /* system file      */
  char group[E_BUFF_SIZE];             
  int CheckCounter;                    /* No of file failing a check        */
  e_string *messages;         /* Result messages                            */
  e_aset *hosts_groups;       /* List of all the host groups                */
  e_aset *config_groups;      /* List of all config groups                  */
  e_aset *configs;            /* List of all the config files               */
  e_aset *hosts;              /* List of selected hosts                     */
  e_aset *arg_configs;        /* List of selected sys file.                 */
  e_aset *this_host_groups;   /* List of groups this host belongs to        */
};

extern struct sysf_par sysfile;

int read_sysf_hosts(char *config_dir,char *hosts_file);
int read_sysf_config(char *config_dir,char *config_file);
e_aset *read_args_hosts(char *hosts_list);
int make_config_group_lists();
int get_this_host_groups(e_string *host);

void mprintf(const char *fmt, ...);
int mail(char *address,char *subject);

void display_config_groups();
void display_all_config_files();
void display_all_configs();
int display_one_group(char *group);
void display_hosts_groups();
void display_host_group(char *group);
void display_configs();


#endif
